@@ -1,61 +1,70 @@
 #!/bin/bash
 
-echo "---
-version: 2.1
-
-orbs:
-  capture-tag: rvla/capture-tag@0.0.2
-
-executors:"
-
 # TODO: make ASAN counterpart of builders
 
 declare -A all_vendors
 vendor=mysql80
-all_vendors[${vendor}_0]="mysql80"
+all_vendors[${vendor}_0]="$vendor"
 all_vendors[${vendor}_1]="mysql:8.0"
 all_vendors[${vendor}_3]="mysqlclient"
 all_vendors[${vendor}_4]="mysqlclient"
 
 vendor=mysql84
-all_vendors[${vendor}_0]="mysql84"
+all_vendors[${vendor}_0]="$vendor"
 all_vendors[${vendor}_1]="mysql:8.4"
 all_vendors[${vendor}_3]="mysqlclient"
 all_vendors[${vendor}_4]="mysqlclient"
 
 vendor=percona57
-all_vendors[${vendor}_0]="percona57"
+all_vendors[${vendor}_0]="$vendor"
 all_vendors[${vendor}_1]="percona:5.7"
 all_vendors[${vendor}_3]="perconaserverclient"
 all_vendors[${vendor}_4]="perconaserverclient"
 
 vendor=percona80
-all_vendors[${vendor}_0]="percona80"
+all_vendors[${vendor}_0]="$vendor"
 all_vendors[${vendor}_1]="percona:8"
 all_vendors[${vendor}_3]="perconaserverclient"
 all_vendors[${vendor}_4]="perconaserverclient"
 
 vendor=mariadb1006
-all_vendors[${vendor}_0]="mariadb1006"
+all_vendors[${vendor}_0]="$vendor"
 all_vendors[${vendor}_1]="mariadb:10.6"
 all_vendors[${vendor}_2]="mariadb-10.6"
 all_vendors[${vendor}_3]="mariadb"
 all_vendors[${vendor}_4]="mariadbclient"
 
 vendor=mariadb1011
-all_vendors[${vendor}_0]="mariadb1011"
+all_vendors[${vendor}_0]="$vendor"
 all_vendors[${vendor}_1]="mariadb:10.11"
 all_vendors[${vendor}_2]="mariadb-10.11"
 all_vendors[${vendor}_3]="mariadb"
 all_vendors[${vendor}_4]="mariadbclient"
 
 vendor=tidb
-all_vendors[${vendor}_0]="tidb"
+all_vendors[${vendor}_0]="$vendor"
 all_vendors[${vendor}_1]="pingcap/tidb"
 
+vendor=debian_default
+all_vendors[${vendor}_0]="$vendor"
+all_vendors[${vendor}_1]="mysql:8.4"
+all_vendors[${vendor}_3]="mariadb"
+all_vendors[${vendor}_4]="mariadbclient"
+
+vendor=ubuntu_default
+all_vendors[${vendor}_0]="$vendor"
+all_vendors[${vendor}_1]="mysql:8.4"
+all_vendors[${vendor}_3]="mysqlclient"
+all_vendors[${vendor}_4]="mysqlclient"
+
+#_0: name
+#_1: docker image
+#_2
+#_3: variable found in CMAKE, like: -DMYSQL_LIBRARIES_${all_vendors[${vendor}_3]}
+#_4: static library name, like: lib${all_vendors[${vendor}_4]}.a
 
 
-list_mysql_version=( "mysql80" "mysql84" )
+list_mysql_version=( "mysql80" "mysql84" "debian_default" "ubuntu_default")
 list_percona_version=( "percona57" "percona80" )
 list_mariadb_version=( "mariadb1011" "mariadb1006")
 list_all_vendors=( "${list_mysql_version[@]}" "${list_percona_version[@]}" "${list_mariadb_version[@]}" )
@@ -133,6 +142,11 @@ all_os[${os}_1]="debian:bookworm"
 all_os[${os}_2]="percona-release_latest.bookworm_all.deb"
 all_os[${os}_3]="true"
 
+os=trixie
+all_os[${os}_0]="trixie"
+all_os[${os}_1]="debian:trixie"
+all_os[${os}_2]="percona-release_latest.trixie_all.deb"
+all_os[${os}_3]="true"
 
 # os=
 # all_os[${os}_0]=""
@@ -140,8 +154,8 @@ all_os[${os}_3]="true"
 list_el_os=("el7" "el8" "el9")
 list_el_os_without_el=( "7" "8" "9" )
 list_ubuntu_os=("bionic" "focal" "jammy" "noble")
-list_debian_os=("buster" "bullseye" "bookworm")
-list_all_os=("bionic" "focal" "jammy" "noble" "el7" "el8" "el9" "buster" "bullseye" "bookworm")
+list_debian_os=("buster" "bullseye" "bookworm" "trixie")
+list_all_os=("bionic" "focal" "jammy" "noble" "el7" "el8" "el9" "buster" "bullseye" "bookworm" "trixie")
 
 build_man_os="jammy_percona80_amd64"
 
@@ -149,31 +163,50 @@ list_build=(
   "bionic_percona80_amd64"   
   "focal_percona80_amd64"   # "focal_mariadb1011_arm64"
   "jammy_percona80_amd64"   # "jammy_mariadb1011_arm64"
-  "noble_mysql84_amd64"
+  "noble_mysql84_amd64"         "noble_ubuntu_default_arm64"
   "el7_percona57_x86_64" 
-  "el8_percona80_x86_64"     "el8_mysql80_aarch64"
-  "el9_percona80_x86_64"     "el9_mysql80_aarch64"
+  "el8_mysql84_x86_64"        "el8_mysql84_aarch64"
+  "el9_mysql84_x86_64"        "el9_mysql84_aarch64"
   "bullseye_percona80_amd64" 
   "buster_percona80_amd64"
-  "bookworm_percona80_amd64" "bookworm_mariadb1011_arm64")
+  "bookworm_mysql84_amd64"    "bookworm_mariadb1011_arm64"
+  "trixie_debian_default_amd64" "trixie_debian_default_arm64"
+)
 
 #   "noble_percona57"    "noble_percona80"    "noble_mariadb1011"    "noble_mariadb1006"
 list_compile=(
   "bionic_percona57"   "bionic_percona80"
   "focal_percona57"    "focal_percona80"    "focal_mariadb1011"    "focal_mariadb1006"
-                                                                                          "noble_mysql84"
+# jammy is in the tests list 
+#                                                                                          "noble_mysql84" This is already on the list of test
+                                                                                                             "noble_ubuntu_default"
   "el7_percona57"      "el7_percona80"      "el7_mariadb1011"      "el7_mariadb1006"      "el7_mysql84"
   "el8_percona57"      "el8_percona80"      "el8_mariadb1011"      "el8_mariadb1006"      "el8_mysql84"
                        "el9_percona80"      "el9_mariadb1011"      "el9_mariadb1006"      "el9_mysql84"
   "buster_percona57"   "buster_percona80"
   "bullseye_percona57" "bullseye_percona80" "bullseye_mariadb1011" "bullseye_mariadb1006"
-  "bookworm_percona57" "bookworm_percona80" "bookworm_mariadb1011"                        "bookworm_mysql84")
+  "bookworm_percona57" "bookworm_percona80" "bookworm_mariadb1011"                        "bookworm_mysql84"
+                                                                                                             "trixie_debian_default"
+
+)
+
+list_test=("jammy_percona57" "jammy_percona80" "jammy_mariadb1011" "jammy_mariadb1006" "noble_mysql84")
+
+echo "---
+version: 2.1
+
+orbs:
+  capture-tag: rvla/capture-tag@0.0.2
+
+executors:"
+
+# EXECUTORS
 
-list_test=("jammy_percona57" "jammy_percona80" "jammy_mariadb1011" "jammy_mariadb1006" "jammy_mysql84")
 
 
 for os in ${list_all_os[@]}
 do
+## EXECUTORS BY VENDOR
     for vendor in ${list_mariadb_version[@]} ${list_percona_version[@]} tidb
     do
         echo "
@@ -210,6 +243,7 @@ do
         MYSQL_ALLOW_EMPTY_PASSWORD: true
     working_directory: /tmp/src/mydumper"
     done
+## SINGLE EXECUTOR: used just for building 
                 echo "
   ${all_os[${os}_0]}:
     docker:
@@ -223,19 +257,24 @@ do
     working_directory: /tmp/src/mydumper"
 done
 
+# COMMANDS
+
+
+## SPECIFIC PREPARE COMMANDS
+
 cat <<EOF
 
 commands:
 
   prepare_mariadb1006:
     steps:
-    - run: sudo bash /tmp/mariadb_repo_setup --mariadb-server-version "mariadb-10.6"
-    - run: yum-config-manager --disable mariadb-maxscale || true
+    - run: sudo bash /tmp/mariadb_repo_setup --skip-maxscale --mariadb-server-version "mariadb-10.6"
+#    - run: yum-config-manager --disable mariadb-maxscale || true
 
   prepare_mariadb1011:
     steps:
-    - run: sudo bash /tmp/mariadb_repo_setup --mariadb-server-version "mariadb-10.11"
-    - run: yum-config-manager --disable mariadb-maxscale || true
+    - run: sudo bash /tmp/mariadb_repo_setup --skip-maxscale --mariadb-server-version "mariadb-10.11"
+#    - run: yum-config-manager --disable mariadb-maxscale || true
 
   prepare_el_mysql80:
     steps:
@@ -245,17 +284,25 @@ commands:
     steps:
     - run: sudo yum install -y libasan gdb screen time mysql-community-libs mysql-community-devel mysql-community-client
 
-  prepare_ubuntu_percona57:
+  prepare_apt_debian_default:
+    steps:
+    - run: sudo apt-get install -y default-mysql-client default-libmysqlclient-dev default-mysql-client-core
+
+  prepare_apt_ubuntu_default:
+    steps:
+    - run: sudo apt-get install -y default-mysql-client default-libmysqlclient-dev default-mysql-client-core
+
+  prepare_apt_percona57:
     steps:
     - run: sudo percona-release setup -y ps57
     - run: sudo apt-get install -y gdb screen time libperconaserverclient20 percona-server-client-5.7 libperconaserverclient20-dev
 
-  prepare_ubuntu_percona80:
+  prepare_apt_percona80:
     steps:
     - run: sudo percona-release setup -y ps80
     - run: sudo apt-get install -y gdb screen time libperconaserverclient21 libperconaserverclient21-dev percona-server-client
 
-  prepare_ubuntu_mysql84:
+  prepare_apt_mysql84:
     steps:
     - run: echo "mysql-apt-config mysql-apt-config/select-product string Ok" | sudo debconf-set-selections
     - run: echo "mysql-apt-config mysql-apt-config/select-server string mysql-8.4-lts" | sudo debconf-set-selections
@@ -264,7 +311,7 @@ commands:
     - run: sudo apt-get update
     - run: sudo apt-get install -y gdb screen time libmysqlclient24 libmysqlclient-dev mysql-client
 
-  prepare_ubuntu_mariadb1006:
+  prepare_apt_mariadb1006:
     steps:
     - run: sudo apt-get install -y gdb screen time mariadb-client libmariadbclient18 libmariadb-dev libmariadb-dev-compat || true
     - run: sudo apt-get install -y gdb screen time mariadb-client libmariadbclient18 libmariadb-dev libmariadb-dev-compat
@@ -275,11 +322,26 @@ commands:
     - run: sudo yum install -y libasan gdb screen time MariaDB-devel
     - run: sudo yum install -y libasan gdb screen time MariaDB-compat || true
 
-  prepare_ubuntu_mariadb1011:
+  prepare_apt_mariadb1011:
     steps:
     - run: sudo apt-get install -y gdb screen time mariadb-client libmariadbclient18 libmariadb-dev libmariadb-dev-compat || true
     - run: sudo apt-get install -y gdb screen time mariadb-client libmariadbclient18 libmariadb-dev libmariadb-dev-compat
 
+
+  prepare_el7_mariadb1011:
+    steps:
+    - run: sudo bash /tmp/mariadb_repo_setup --skip-maxscale --mariadb-server-version "mariadb-10.11.11"
+#    - run: yum-config-manager --disable mariadb-maxscale || true
+    - run: sudo yum install -y libasan gdb screen time MariaDB-devel
+    - run: sudo yum install -y libasan gdb screen time MariaDB-compat || true
+
+  prepare_el7_mariadb1006:
+    steps:
+    - run: sudo bash /tmp/mariadb_repo_setup --skip-maxscale --mariadb-server-version "mariadb-10.6.21"
+#    - run: yum-config-manager --disable mariadb-maxscale || true
+    - run: sudo yum install -y libasan gdb screen time MariaDB-devel
+    - run: sudo yum install -y libasan gdb screen time MariaDB-compat || true
+
   prepare_el_mariadb1011:
     steps:
     - prepare_mariadb1011
@@ -309,7 +371,7 @@ do
 "
 done
 
-for os in ${list_el_os[@]}
+for os in el8 el9
 do
     for vendor in ${list_mariadb_version[@]}
     do
@@ -322,6 +384,7 @@ do
 done
 
 
+
 # On apt repositories OS the preparation
 for os in ${list_ubuntu_os[@]} ${list_debian_os[@]}
 do
@@ -330,7 +393,7 @@ do
     echo "
   prepare_${all_os[${os}_0]}_${vendor}:
     steps:
-    - prepare_ubuntu_percona57
+    - prepare_apt_percona57
 "
     # For Percona and MySQL will be the standar apt preparation
     for vendor in ${list_percona_version[@]} ${list_mysql_version[@]}
@@ -338,7 +401,7 @@ do
 echo "
   prepare_${all_os[${os}_0]}_${all_vendors[${vendor}_0]}:
     steps:
-    - prepare_ubuntu_${all_vendors[${vendor}_0]}
+    - prepare_apt_${all_vendors[${vendor}_0]}
 "
     done
 
@@ -349,7 +412,7 @@ echo "
   prepare_${all_os[${os}_0]}_${all_vendors[${vendor}_0]}:
     steps:
     - prepare_${all_vendors[${vendor}_0]}
-    - prepare_ubuntu_${all_vendors[${vendor}_0]}
+    - prepare_apt_${all_vendors[${vendor}_0]}
 "
     done
 done
@@ -405,9 +468,11 @@ cat <<EOF
     steps:
     - run:
         command: |
+          if [ -z \${CIRCLE_TAG+x} ] ; then echo 'export CIRCLE_TAG="v0.11.1-1"' >> "\$BASH_ENV"; fi
           echo 'export MYDUMPER_VERSION=\$(  echo "\${CIRCLE_TAG:1}" | cut -d'-' -f1 ) ' >> "\$BASH_ENV"
           echo 'export MYDUMPER_REVISION=\$( echo "\${CIRCLE_TAG:1}" | cut -d'-' -f2 ) ' >> "\$BASH_ENV"
           cat /etc/profile.d/sh.local >> "\$BASH_ENV" || true
+          cat \$BASH_ENV
           source "\$BASH_ENV"
 
 jobs:
@@ -431,7 +496,6 @@ do
     resource_class: large
     steps:
     - checkout
-#    - prepare_ubuntu
     - prepare_${all_os[${os}_0]}_${all_vendors[${vendor}_0]}
     - compile_and_test_mydumper:
         test: << parameters.test >>
@@ -461,7 +525,6 @@ do
     resource_class: large
     steps:
     - checkout
-#    - prepare_el
     - prepare_${all_os[${os}_0]}_${all_vendors[${vendor}_0]}
     - compile_and_test_mydumper:
         test: << parameters.test >>
@@ -481,7 +544,6 @@ do
         for vendor in ${list_all_vendors[@]}
         do
 echo "  build_${all_os[${os}_0]}_${all_vendors[${vendor}_0]}_${all_arch[${arch}_rpm]}:
-#    executor: ${all_os[${os}_0]}_${all_vendors[${vendor}_0]}
     executor: ${all_os[${os}_0]}
     resource_class: ${all_arch[${arch}_resource_class]}
     steps:
@@ -492,7 +554,6 @@ echo '    - attach_workspace:
         at: /tmp/man'
 fi
 echo "    - set_env_vars
-#    - prepare_el
     - prepare_${os}_${all_vendors[${vendor}_0]}
     - run: mkdir -p /tmp/package
     - run: yum -y install rpmdevtools
@@ -502,6 +563,13 @@ echo "    - set_env_vars
     - run: mkdir -p /tmp/src/mydumper/${all_os[${os}_0]}_${all_vendors[${vendor}_0]}_${all_arch[${arch}_rpm]}
     - run: cp /tmp/man/mydumper.1.gz /tmp/man/myloader.1.gz mydumper.cnf mydumper myloader /tmp/src/mydumper/${all_os[${os}_0]}_${all_vendors[${vendor}_0]}_${all_arch[${arch}_rpm]}/
     - run: ./package/build.sh \${MYDUMPER_VERSION} \${MYDUMPER_REVISION} rpm ${all_os[${os}_0]}_${all_vendors[${vendor}_0]}_${all_arch[${arch}_rpm]} ${all_arch[${arch}_rpm]}"
+#        command: |
+#          if [ -z ${CIRCLE_TAG+x} ];
+#          then
+#            export MYDUMPER_VERSION=\"9.9.9\"
+#            export MYDUMPER_REVISION=\"9\"
+#          fi
+#          ./package/build.sh \${MYDUMPER_VERSION} \${MYDUMPER_REVISION} rpm ${all_os[${os}_0]}_${all_vendors[${vendor}_0]}_${all_arch[${arch}_rpm]} ${all_arch[${arch}_rpm]}"
 echo "    - persist_to_workspace:
          root: /tmp/package
          paths:
@@ -524,7 +592,6 @@ do
         for vendor in ${list_all_vendors[@]}
         do
 echo "  build_${all_os[${os}_0]}_${all_vendors[${vendor}_0]}_${all_arch[${arch}_deb]}:
-#    executor: ${all_os[${os}_0]}_${all_vendors[${vendor}_0]}
     executor: ${all_os[${os}_0]}
     resource_class: ${all_arch[${arch}_resource_class]}
     parameters:
@@ -532,7 +599,6 @@ echo "  build_${all_os[${os}_0]}_${all_vendors[${vendor}_0]}_${all_arch[${arch}_
         type: boolean
         default: false
     
-
     steps:
     - checkout"
 if [ "${all_os[${os}_0]}_${all_vendors[${vendor}_0]}_${all_arch[${arch}_deb]}" != "${build_man_os}" ]
@@ -541,7 +607,6 @@ echo '    - attach_workspace:
         at: /tmp/man'
 fi
 echo "    - set_env_vars
-#    - prepare_ubuntu
     - prepare_${all_os[${os}_0]}_${all_vendors[${vendor}_0]}
     - run: sudo apt install -y fakeroot
     - run: mkdir -p /tmp/man/
@@ -564,6 +629,13 @@ echo "    - set_env_vars
     - run: mkdir -p /tmp/src/mydumper/${all_os[${os}_0]}_${all_vendors[${vendor}_0]}_${all_arch[${arch}_deb]}/etc
     - run: cp /tmp/man/mydumper.1.gz /tmp/man/myloader.1.gz  mydumper.cnf mydumper myloader /tmp/src/mydumper/${all_os[${os}_0]}_${all_vendors[${vendor}_0]}_${all_arch[${arch}_deb]}/
     - run: ./package/build.sh \${MYDUMPER_VERSION} \${MYDUMPER_REVISION} deb ${all_os[${os}_0]}_${all_vendors[${vendor}_0]}_${all_arch[${arch}_deb]} ${all_arch[${arch}_deb]}"
+#        command: |
+#          if [ -z ${CIRCLE_TAG+x} ];
+#          then
+#            export MYDUMPER_VERSION=\"9.9.9\"
+#            export MYDUMPER_REVISION=\"9\"
+#          fi
+#          ./package/build.sh \${MYDUMPER_VERSION} \${MYDUMPER_REVISION} deb ${all_os[${os}_0]}_${all_vendors[${vendor}_0]}_${all_arch[${arch}_deb]} ${all_arch[${arch}_deb]}"
 echo '    - persist_to_workspace:
          root: /tmp/package
          paths:
@@ -589,7 +661,7 @@ echo '
     - run:
         command: |
           rm /tmp/package/mydumper.1.gz /tmp/package/myloader.1.gz
-          ghr -t ${GITHUB_TOKEN} -u ${CIRCLE_PROJECT_USERNAME} -r ${CIRCLE_PROJECT_REPONAME} -c ${CIRCLE_SHA1} -b "$( cd /tmp/package/; echo -e "MD5s:\n\`\`\`"; md5sum * ;echo -e "\n\`\`\`\nSHA1s:\n\`\`\`"; sha1sum * ; echo -e "\`\`\`\nSHA256s:\n\`\`\`"; sha256sum * ;echo -e "\`\`\`\n" )" -prerelease -draft -delete ${CIRCLE_TAG} /tmp/package'
+          ghr -t ${GITHUB_TOKEN} -u ${CIRCLE_PROJECT_USERNAME} -r ${CIRCLE_PROJECT_REPONAME} -c ${CIRCLE_SHA1} -b "$( cd /tmp/package/; echo -e "MD5s:\n\`\`\`"; md5sum * ;echo -e "\`\`\`\nSHA1s:\n\`\`\`"; sha1sum * ; echo -e "\`\`\`\nSHA256s:\n\`\`\`"; sha256sum * ;echo -e "\`\`\`\n" )" -prerelease -draft -delete ${CIRCLE_TAG} /tmp/package'
 
 echo -n '
   publish-repository:
@@ -680,23 +752,69 @@ echo '
           git update-index --cacheinfo 160000,$(head -2 ../latest_commit | tail -1 | cut -b9-48),repo
           git commit -am "[skip ci] Auto updated submodule references" && git push
 
+  update_repo:
+    docker:
+      - image: mydumper/mydumper-builder-noble
+    steps:
+    - run:
+        command: |
+          git config --global user.name "David Ducos"
+          git config --global user.email "david.ducos@gmail.com"
+          git clone https://github.com/mydumper/mydumper.git mydumper
+          cd mydumper
+          git remote set-url origin https://x-access-token:${GITHUB_TOKEN}@github.com/mydumper/mydumper.git
+          curl https://github.com/mydumper/mydumper_repo/info/refs?service=git-upload-pack --output ../latest_commit
+          git update-index --cacheinfo 160000,$(head -2 ../latest_commit | tail -1 | cut -b9-48),repo
+          git commit -am "[skip ci] Auto updated submodule references" && git push
+
+
+
+parameters:
+  my_trigger_parameter:
+    type: string
+    default: ""
+
 
 workflows:
   version: 2
+
+  api-update-repo:
+    when: << pipeline.parameters.my_trigger_parameter >>
+    jobs:
+    - update_repo
+
+#  api-update-repo-commit:
+#    when: 
+#      not: << pipeline.parameters.my_trigger_parameter >>
+#    jobs:
+#    - update_repo
+
   mydumper:
     jobs:'
 
 for lc in ${!list_compile[@]}
 do
 echo "    - compile_and_test_mydumper_in_${list_compile[${lc}]}"
+# Decomment next 5 lines if you want to ignore compilation and add : to previous line
+#echo '        filters:
+#          branches:
+#            ignore: /.*/
+#          tags:
+#            only: /^v\d+\.\d+\.\d+-\d+$/'
 done
 
 for lt in ${!list_test[@]}
 do
 echo "    - compile_and_test_mydumper_in_${list_test[${lt}]}:
         test: true
-        e: ${list_test[${lt}]}
-"
+        e: ${list_test[${lt}]}"
+# Decomment next 5 lines if you want to ignore compilation
+#echo '        filters:
+#          branches:
+#            ignore: /.*/
+#          tags:
+#            only: /^v\d+\.\d+\.\d+-\d+$/'
+
 done
 
 for os in ${list_build[@]}
@@ -709,6 +827,8 @@ else
 echo "        requires:
           - build_${build_man_os}"
 fi
+
+# Comment next 5 lines if you want to build
 echo '        filters:
           branches:
             ignore: /.*/
