@@ -54,8 +54,6 @@ gboolean skip_indexes = FALSE;
 // For daemon mode
 gboolean shutdown_triggered = FALSE;
 
-guint errors;
-
 void parse_disk_limits(){
   gchar ** strsplit = g_strsplit(disk_limits,":",3);
   if (g_strv_length(strsplit)!=2){
@@ -72,8 +70,6 @@ int main(int argc, char *argv[]) {
   setlocale(LC_ALL, "");
   context = load_contex_entries();
 
-  initialize_share_common();
-
   gchar ** tmpargv=g_strdupv(argv);
   int tmpargc=argc;
   if (!g_option_context_parse(context, &tmpargc, &tmpargv, &error)) {
@@ -239,6 +235,7 @@ int main(int argc, char *argv[]) {
     print_string("outputdir",output_directory);
     print_bool("clear",clear_dumpdir);
     print_bool("dirty",dirty_dumpdir);
+    print_bool("merge",merge_dumpdir);
     print_bool("stream",stream);
     print_string("logfile",logfile);
     print_string("disk-limits",disk_limits);
