@@ -32,11 +32,7 @@
 /* Program options */
 gboolean dump_triggers = FALSE;
 gboolean ignore_generated_fields = FALSE;
-gchar *exec_per_thread = NULL;
-const gchar *exec_per_thread_extension = NULL;
-gchar **exec_per_thread_cmd=NULL;
 gboolean skip_definer = FALSE;
-gboolean success_on_1146 = FALSE;
 
 extern gchar *table_engine_for_view_dependency;
 
@@ -48,17 +44,6 @@ void initialize_jobs(){
   if (ignore_generated_fields)
     g_warning("Queries related to generated fields are not going to be executed. It will lead to restoration issues if you have generated columns");
 
-  if (exec_per_thread_extension != NULL && strlen(exec_per_thread_extension)>0){
-    if(exec_per_thread == NULL)
-      m_error("--exec-per-thread needs to be set when --exec-per-thread-extension (%s) is used", exec_per_thread_extension);
-  }
-
-  if (exec_per_thread!=NULL){
-    if (exec_per_thread[0]!='/'){
-      m_error("Absolute path is only allowed when --exec-per-thread is used");
-    }
-    exec_per_thread_cmd=g_strsplit(exec_per_thread, " ", 0);
-  }
 }
 
 static
@@ -67,6 +52,7 @@ gchar * write_checksum_into_file(MYSQL *conn, struct database *database, char *t
   gchar *checksum=fun(conn, database->name, table, &errn);
 //  g_message("Checksum value: %s", checksum);
   if (errn != 0 && !(success_on_1146 && errn == 1146)) {
+    g_warning("Writing checksum");
     errors++;
     return NULL;
   }
@@ -89,7 +75,6 @@ gchar * get_tablespace_query(){
 static
 void write_tablespace_definition_into_file(MYSQL *conn,char *filename){
   char *query = NULL;
-  MYSQL_RES *result = NULL;
   MYSQL_ROW row;
   int outfile = m_open(&filename,"w");
   if (!outfile) {
@@ -103,25 +88,16 @@ void write_tablespace_definition_into_file(MYSQL *conn,char *filename){
     g_warning("Tablespace resquested, but not possible due to server version not supported");
     return;
   }
-  if (mysql_query(conn, query) || !(result = mysql_use_result(conn))) {
-    if (success_on_1146 && mysql_errno(conn) == 1146) {
-      g_warning("Error dumping create tablespace: %s",
-                mysql_error(conn));
-    } else {
-      g_critical("Error dumping create tablespace: %s",
-                 mysql_error(conn));
-      errors++;
-    }
-    g_free(query);
+  MYSQL_RES *result = m_store_result_critical(conn, query,  "Error dumping create tablespace", NULL);
+  g_free(query);
+  if (!result)
     return;
-  }
 
-  const char q= identifier_quote_character;
   GString *statement = g_string_sized_new(statement_size);
   initialize_sql_statement(statement);
 
   while ((row = mysql_fetch_row(result))) {
-    g_string_append_printf(statement, "CREATE TABLESPACE %c%s%c ADD DATAFILE '%s' FILE_BLOCK_SIZE = %s ENGINE=INNODB;\n", q, row[0], q, row[1], row[2]);
+    g_string_append_printf(statement, "CREATE TABLESPACE %c%s%c ADD DATAFILE '%s' FILE_BLOCK_SIZE = %s ENGINE=INNODB;\n", identifier_quote_character, row[0], identifier_quote_character, row[1], row[2]);
     if (!write_data(outfile, statement)) {
       g_critical("Could not write tablespace data for %s", row[0]);
       errors++;
@@ -129,14 +105,12 @@ void write_tablespace_definition_into_file(MYSQL *conn,char *filename){
     }
     g_string_set_size(statement, 0);
   }
+  g_string_free(statement, TRUE);
 }
 
 static
 void write_schema_definition_into_file(MYSQL *conn, struct database *database, char *filename) {
   int outfile=0;
-  char *query = NULL;
-  MYSQL_RES *result = NULL;
-  MYSQL_ROW row;
 
   outfile = m_open(&filename,"w");
 
@@ -147,43 +121,31 @@ void write_schema_definition_into_file(MYSQL *conn, struct database *database, c
     return;
   }
 
-  const char q= identifier_quote_character;
   GString *statement = g_string_sized_new(statement_size);
   initialize_sql_statement(statement);
-
-  query = g_strdup_printf("SHOW CREATE DATABASE IF NOT EXISTS %c%s%c", q, database->name, q);
-  if (mysql_query(conn, query) || !(result = mysql_use_result(conn))) {
-    if (success_on_1146 && mysql_errno(conn) == 1146) {
-      g_warning("Error dumping create database (%s): %s", database->name,
-                mysql_error(conn));
-    } else {
-      g_critical("Error dumping create database (%s): %s", database->name,
-                 mysql_error(conn));
-      errors++;
-    }
-    g_free(query);
+  char *query = g_strdup_printf("SHOW CREATE DATABASE IF NOT EXISTS %c%s%c", identifier_quote_character, database->name, identifier_quote_character);
+  struct M_ROW *mr = m_store_result_row (conn, query, m_critical, m_warning, "Error dumping create database (%s)", database->name);
+  g_free(query);
+  if (!mr->res){
+    m_store_result_row_free(mr);
     return;
   }
 
   /* There should never be more than one row */
-  row = mysql_fetch_row(result);
-  if (!strstr(row[1], identifier_quote_character_str)) {
+  if (!mr->row || !strstr(mr->row[1], identifier_quote_character_str)) {
     g_critical("Identifier quote [%s] not found when fetching %s",
                identifier_quote_character_str, database->name);
     errors++;
   }
-  g_string_append(statement, row[1]);
+  g_string_append(statement, mr->row[1]);
   g_string_append(statement, ";\n");
   if (!write_data(outfile, statement)) {
     g_critical("Could not write create database for %s", database->name);
     errors++;
   }
-  g_free(query);
   m_close(0, outfile, filename, 1, NULL);
   g_string_free(statement, TRUE);
-  if (result)
-    mysql_free_result(result);
-
+  m_store_result_row_free(mr);
 
   if (schema_checksums)
     database->schema_checksum = write_checksum_into_file(conn, database, NULL, checksum_database_defaults);
@@ -195,8 +157,6 @@ void write_table_definition_into_file(MYSQL *conn, struct db_table *dbt,
                       char *filename, gboolean checksum_filename, gboolean checksum_index_filename) {
   int outfile;
   char *query = NULL;
-  MYSQL_RES *result = NULL;
-  MYSQL_ROW row;
   outfile = m_open(&filename,"w");
 
   if (!outfile) {
@@ -217,35 +177,24 @@ void write_table_definition_into_file(MYSQL *conn, struct db_table *dbt,
   }
 
   query = g_strdup_printf("SHOW CREATE TABLE %c%s%c.%c%s%c", q, dbt->database->name, q, q, dbt->table, q);
-  if (mysql_query(conn, query) || !(result = mysql_use_result(conn))) {
-    if (success_on_1146 && mysql_errno(conn) == 1146) {
-      g_warning("Error dumping schemas (%s.%s): %s", dbt->database->name, dbt->table,
-                mysql_error(conn));
-    } else {
-      g_critical("Error dumping schemas (%s.%s): %s", dbt->database->name, dbt->table,
-                 mysql_error(conn));
-      errors++;
-    }
-    g_free(query);
+  struct M_ROW *mr = m_store_result_row(conn, query, m_critical, m_warning, "Error dumping schemas (%s.%s)", dbt->database->name, dbt->table);
+  g_free(query);
+  if (!mr->res){
+    m_store_result_row_free(mr);
     return;
   }
 
   g_string_set_size(statement, 0);
 
-  /* There should never be more than one row */
-  row = mysql_fetch_row(result);
-
-  char *create_table;
-  if (schema_sequence_fix) {
-    create_table = filter_sequence_schemas(row[1]);
-  } else {
-    create_table = row[1];
-  }
-
-  g_string_append(statement, create_table);
   if (schema_sequence_fix) {
+    gchar *create_table=NULL;
+    g_string_append(statement, create_table = filter_sequence_schemas(mr->row[1]));
     g_free(create_table);
+  } else {
+    g_string_append(statement, mr->row[1]);
   }
+  m_store_result_row_free(mr);
+
   g_string_append(statement, ";\n");
 
   if (skip_indexes || skip_constraints){
@@ -267,31 +216,25 @@ void write_table_definition_into_file(MYSQL *conn, struct db_table *dbt,
       errors++;
     }
   }
-  g_free(query);
   m_close(0, outfile, filename, 1, dbt);
   g_string_free(statement, TRUE);
-  if (result)
-    mysql_free_result(result);
 
-  if (checksum_filename){
+  if (checksum_filename)
     dbt->schema_checksum=write_checksum_into_file(conn, dbt->database, dbt->table, checksum_table_structure);
-//    g_message("Checksum for table schema: %s", dbt->schema_checksum);
-  }
-  if (checksum_index_filename){
+  
+  if (checksum_index_filename)
     dbt->indexes_checksum=write_checksum_into_file(conn, dbt->database, dbt->table, checksum_table_indexes);
-  }
+  
   return;
 }
 
 static
 void write_triggers_definition_into_file(MYSQL *conn, MYSQL_RES *result, struct database *database, gchar *message, int outfile) {
-  MYSQL_RES *result2 = NULL;
-  MYSQL_ROW row2;
   MYSQL_ROW row;
   gchar *query = NULL;
   gchar **splited_st = NULL;
-  const char q= identifier_quote_character;
   GString *statement = g_string_sized_new(statement_size);
+  GString *create_trigger = g_string_sized_new(statement_size);
   initialize_sql_statement(statement);
 
   if (!write_data(outfile, statement)) {
@@ -308,26 +251,35 @@ void write_triggers_definition_into_file(MYSQL *conn, MYSQL_RES *result, struct
       return;
     }
     g_string_set_size(statement, 0);
-    query = g_strdup_printf("SHOW CREATE TRIGGER %c%s%c.%c%s%c", q, database->name, q, q, row[0], q);
-    mysql_query(conn, query);
-    result2 = mysql_store_result(conn);
-    row2 = mysql_fetch_row(result2);
-    if ( skip_definer && g_str_has_prefix(row2[2],"CREATE")){
-      remove_definer_from_gchar(row2[2]);
-    }
-    g_string_append_printf(statement, "%s", row2[2]);
-    splited_st = g_strsplit(statement->str, ";\n", 0);
-    g_string_printf(statement, "%s", g_strjoinv("; \n", splited_st));
-    g_strfreev(splited_st);
-    g_string_append(statement, ";\n");
-    restore_charset(statement);
-    if (!write_data(outfile, statement)) {
-      g_critical("Could not write triggers data for %s", message);
-      errors++;
-      return;
+    struct M_ROW *mr = m_store_result_single_row(conn, query = g_strdup_printf("SHOW CREATE TRIGGER %c%s%c.%c%s%c", 
+                        identifier_quote_character, database->name, identifier_quote_character, 
+                        identifier_quote_character, row[0], identifier_quote_character),
+                        "Failed to execute SHOW CREATE TRIGGER %s.%s",database->name, row[0] );
+    g_free(query);
+    if (mr->row){
+      if ( skip_definer && g_str_has_prefix(mr->row[2],"CREATE"))
+        remove_definer_from_gchar(mr->row[2]);
+      g_string_append_printf(statement, "DROP TRIGGER IF EXISTS %c%s%c;\n",
+                        identifier_quote_character, row[0], identifier_quote_character);
+      g_string_set_size(create_trigger, 0);
+      g_string_append_printf(create_trigger, "%s", mr->row[2]);
+      splited_st = g_strsplit(create_trigger->str, ";\n", 0);
+      g_string_printf(create_trigger, "%s", g_strjoinv("; \n", splited_st));
+      g_strfreev(splited_st);
+      g_string_append(statement, create_trigger->str);
+      g_string_append(statement, ";\n");
+      restore_charset(statement);
+      if (!write_data(outfile, statement)) {
+        g_critical("Could not write triggers data for %s", message);
+        errors++;
+        return;
+      }
     }
+    m_store_result_row_free(mr);
     g_string_set_size(statement, 0);
   }
+  g_string_free(create_trigger, TRUE);
+  g_string_free(statement, TRUE);
   return;
 }
 
@@ -335,7 +287,6 @@ static
 void write_triggers_definition_into_file_from_dbt(MYSQL *conn, struct db_table *dbt, char *filename, gboolean checksum_filename) {
   int outfile;
   char *query = NULL;
-  MYSQL_RES *result = NULL;
 
   outfile = m_open(&filename,"w");
 
@@ -347,22 +298,11 @@ void write_triggers_definition_into_file_from_dbt(MYSQL *conn, struct db_table *
   }
 
   // get triggers
-  const char q= identifier_quote_character;
-  query = g_strdup_printf("SHOW TRIGGERS FROM %c%s%c WHERE %cTable%c = '%s'", q, dbt->database->name, q, q, q, dbt->table);
-
-  if (mysql_query(conn, query) || !(result = mysql_store_result(conn))) {
-    if (success_on_1146 && mysql_errno(conn) == 1146) {
-      g_warning("Error dumping triggers (%s.%s): %s", dbt->database->name, dbt->table,
-                mysql_error(conn));
-    } else {
-      g_critical("Error dumping triggers (%s.%s): %s", dbt->database->name, dbt->table,
-                 mysql_error(conn));
-      errors++;
-    }
-    g_free(query);
-    return;
-  }
+  query = g_strdup_printf("SHOW TRIGGERS FROM %c%s%c WHERE %cTable%c = '%s'", identifier_quote_character, dbt->database->name, identifier_quote_character,identifier_quote_character,identifier_quote_character, dbt->table);
+  MYSQL_RES *result = m_store_result_critical(conn, query,  "Error dumping triggers (%s.%s)", dbt->database->name, dbt->table);
   g_free(query);
+  if (!result)
+    return;
 
   gchar *message=g_strdup_printf("%s.%s",dbt->database->name, dbt->table);
   write_triggers_definition_into_file(conn, result, dbt->database, message, outfile);
@@ -377,11 +317,7 @@ void write_triggers_definition_into_file_from_dbt(MYSQL *conn, struct db_table *
 }
 
 void write_triggers_definition_into_file_from_database(MYSQL *conn, struct database *database, char *filename, gboolean checksum_filename) {
-  int outfile;
-  char *query = NULL;
-  MYSQL_RES *result = NULL;
-
-  outfile = m_open(&filename,"w");
+  int outfile = m_open(&filename,"w");
 
   if (!outfile) {
     g_critical("Error: DB: %s Could not create output file %s (%d)", database->name,
@@ -391,39 +327,24 @@ void write_triggers_definition_into_file_from_database(MYSQL *conn, struct datab
   }
 
   // get triggers
-  const char q= identifier_quote_character;
-  query = g_strdup_printf("SHOW TRIGGERS FROM %c%s%c", q, database->name, q);
-  if (mysql_query(conn, query) || !(result = mysql_store_result(conn))) {
-    if (success_on_1146 && mysql_errno(conn) == 1146) {
-      g_warning("Error dumping triggers (%s): %s", database->name,
-                mysql_error(conn));
-    } else {
-      g_critical("Error dumping triggers (%s): %s", database->name,
-                 mysql_error(conn));
-      errors++;
-    }
-    g_free(query);
-    return;
-  }
+  gchar *query = g_strdup_printf("SHOW TRIGGERS FROM %c%s%c", identifier_quote_character, database->name, identifier_quote_character);
+  MYSQL_RES *result = m_store_result_critical(conn, query,  "Error dumping triggers (%s)", database->name);
   g_free(query);
-
-  write_triggers_definition_into_file(conn, result, database, database->name, outfile);
-
-  m_close(0, outfile, filename, 1, NULL);
-  if (result)
+  if (result){
+    write_triggers_definition_into_file(conn, result, database, database->name, outfile);
     mysql_free_result(result);
-  if (checksum_filename)
-    database->triggers_checksum=write_checksum_into_file(conn, database, NULL, checksum_trigger_structure_from_database);
+    m_close(0, outfile, filename, 1, NULL);
+    if (checksum_filename)
+      database->triggers_checksum=write_checksum_into_file(conn, database, NULL, checksum_trigger_structure_from_database);
+  }
   return;
 }
 
 static
-void write_view_definition_into_file(MYSQL *conn, struct db_table *dbt, char *filename, char *filename2, gboolean checksum_filename) {
+void write_view_definition_into_file(MYSQL *conn, struct db_table *dbt, char *tmp_table_filename, char *view_filename, gboolean checksum_filename) {
   int outfile;
   char *query = NULL;
-  MYSQL_RES *result = NULL;
   MYSQL_ROW row;
-  const char q= identifier_quote_character;
   GString *statement = g_string_sized_new(statement_size);
   initialize_sql_statement(statement);
 
@@ -434,9 +355,7 @@ void write_view_definition_into_file(MYSQL *conn, struct db_table *dbt, char *fi
     return;
   }
 
-  outfile = m_open(&filename,"w");
-
-
+  outfile = m_open(&tmp_table_filename,"w");
   if (!outfile) {
     g_critical("Error: DB: %s Could not create output file (%d)", dbt->database->name,
                errno);
@@ -450,29 +369,21 @@ void write_view_definition_into_file(MYSQL *conn, struct db_table *dbt, char *fi
     return;
   }
 
-  // we create tables as workaround
+  // We create files with CREATE TABLE statements as workaround
   // for view dependencies
-  query = g_strdup_printf("SHOW FIELDS FROM %c%s%c.%c%s%c", q, dbt->database->name, q, q, dbt->table, q);
-  if (mysql_query(conn, query) || !(result = mysql_use_result(conn))) {
-    if (success_on_1146 && mysql_errno(conn) == 1146) {
-      g_warning("Error dumping schemas (%s.%s): %s", dbt->database->name, dbt->table,
-                mysql_error(conn));
-    } else {
-      g_critical("Error dumping schemas (%s.%s): %s", dbt->database->name, dbt->table,
-                 mysql_error(conn));
-      errors++;
-    }
-    g_free(query);
-    return;
-  }
+  query = g_strdup_printf("SHOW FIELDS FROM %c%s%c.%c%s%c", identifier_quote_character, dbt->database->name, identifier_quote_character, identifier_quote_character, dbt->table, identifier_quote_character);
+  MYSQL_RES *result = m_store_result_critical(conn, query,  "Error dumping schemas (%s.%s)", dbt->database->name, dbt->table);
   g_free(query);
+  if (!result) 
+    return;
+
   g_string_set_size(statement, 0);
-  g_string_append_printf(statement, "CREATE TABLE IF NOT EXISTS %c%s%c(\n", q, dbt->table, q);
+  g_string_append_printf(statement, "CREATE TABLE IF NOT EXISTS %c%s%c(\n", identifier_quote_character, dbt->table, identifier_quote_character);
   row = mysql_fetch_row(result);
-  g_string_append_printf(statement, "%c%s%c int", q, row[0], q);
+  g_string_append_printf(statement, "%c%s%c int", identifier_quote_character, row[0], identifier_quote_character);
   while ((row = mysql_fetch_row(result))) {
     g_string_append(statement, ",\n");
-    g_string_append_printf(statement, "%c%s%c int", q, row[0], q);
+    g_string_append_printf(statement, "%c%s%c int", identifier_quote_character, row[0], identifier_quote_character);
   }
   g_string_append(statement, "\n) ENGINE=");
   g_string_append(statement, table_engine_for_view_dependency);
@@ -480,7 +391,7 @@ void write_view_definition_into_file(MYSQL *conn, struct db_table *dbt, char *fi
     g_string_append(statement," ENCRYPTION='N'");
   g_string_append(statement,";\n");
 
-  if (result)
+  if (result) // should always be true
     mysql_free_result(result);
 
   if (!write_data(outfile, statement)) {
@@ -488,27 +399,20 @@ void write_view_definition_into_file(MYSQL *conn, struct db_table *dbt, char *fi
     errors++;
   }
 
+  m_close(0, outfile, tmp_table_filename, 1, dbt);
+  g_string_set_size(statement, 0);
+
   // real view
-  query = g_strdup_printf("SHOW CREATE VIEW %c%s%c.%c%s%c", q, dbt->database->name, q, q, dbt->table, q);
-  if (mysql_query(conn, query) || !(result = mysql_use_result(conn))) {
-    if (success_on_1146 && mysql_errno(conn) == 1146) {
-      g_warning("Error dumping schemas (%s.%s): %s", dbt->database->name, dbt->table,
-                mysql_error(conn));
-    } else {
-      g_critical("Error dumping schemas (%s.%s): %s", dbt->database->name, dbt->table,
-                 mysql_error(conn));
-      errors++;
-    }
-    g_free(query);
+  query = g_strdup_printf("SHOW CREATE VIEW %c%s%c.%c%s%c", identifier_quote_character, dbt->database->name, identifier_quote_character, identifier_quote_character, dbt->table, identifier_quote_character);
+  struct M_ROW *mr = m_store_result_single_row(conn, query, "Error dumping view (%s.%s)", dbt->database->name, dbt->table);
+  g_free(query);
+  if (!mr->res || !mr->row){
+    m_store_result_row_free(mr);
     return;
   }
 
-  m_close(0, outfile, filename, 1, dbt);
-  g_string_set_size(statement, 0);
-
-  int outfile2;
-  outfile2 = m_open(&filename2,"w");
-  if (!outfile2) {
+  outfile = m_open(&view_filename,"w");
+  if (!outfile) {
     g_critical("Error: DB: %s Could not create output file (%d)", dbt->database->name,
                errno);
     errors++;
@@ -516,10 +420,10 @@ void write_view_definition_into_file(MYSQL *conn, struct db_table *dbt, char *fi
   }
 
   initialize_sql_statement(statement);
-  g_string_append_printf(statement, "DROP TABLE IF EXISTS %c%s%c;\n", q, dbt->table, q);
-  g_string_append_printf(statement, "DROP VIEW IF EXISTS %c%s%c;\n", q, dbt->table, q);
+  g_string_append_printf(statement, "DROP TABLE IF EXISTS %c%s%c;\n", identifier_quote_character, dbt->table, identifier_quote_character);
+  g_string_append_printf(statement, "DROP VIEW IF EXISTS %c%s%c;\n", identifier_quote_character, dbt->table, identifier_quote_character);
 
-  if (!write_data(outfile2, statement)) {
+  if (!write_data(outfile, statement)) {
     g_critical("Could not write schema data for %s.%s", dbt->database->name, dbt->table);
     errors++;
     return;
@@ -527,28 +431,23 @@ void write_view_definition_into_file(MYSQL *conn, struct db_table *dbt, char *fi
 
   g_string_set_size(statement, 0);
 
-  /* There should never be more than one row */
-  row = mysql_fetch_row(result);
-  set_charset(statement, row[2], row[3]);
-  if ( skip_definer && g_str_has_prefix(row[1],"CREATE")){
-    remove_definer_from_gchar(row[1]);
+  set_charset(statement, mr->row[2], mr->row[3]);
+  if ( skip_definer && g_str_has_prefix(mr->row[1],"CREATE")){
+    remove_definer_from_gchar(mr->row[1]);
   }
-  g_string_append(statement, row[1]);
+  g_string_append(statement, mr->row[1]);
   g_string_append(statement, ";\n");
   restore_charset(statement);
-  if (!write_data(outfile2, statement)) {
+  if (!write_data(outfile, statement)) {
     g_critical("Could not write schema for %s.%s", dbt->database->name, dbt->table);
     errors++;
   }
-  g_free(query);
 
-  m_close(0, outfile2, filename2, 1, dbt);
+  m_close(0, outfile, view_filename, 1, dbt);
   g_string_free(statement, TRUE);
-  if (result)
-    mysql_free_result(result);
+  m_store_result_row_free(mr);
 
   if (checksum_filename)
-    // build_meta_filename(database,table,"schema-view-checksum"),
     dbt->schema_checksum=write_checksum_into_file(conn, dbt->database, dbt->table, checksum_view_structure);
   return;
 }
@@ -557,8 +456,6 @@ static
 void write_sequence_definition_into_file(MYSQL *conn, struct db_table *dbt, char *filename, gboolean checksum_filename) {
   int outfile;
   char *query = NULL;
-  MYSQL_RES *result = NULL;
-  MYSQL_ROW row;
   const char q= identifier_quote_character;
   GString *statement = g_string_sized_new(statement_size);
   initialize_sql_statement(statement);
@@ -585,69 +482,46 @@ void write_sequence_definition_into_file(MYSQL *conn, struct db_table *dbt, char
   }
 
   query = g_strdup_printf("SHOW CREATE SEQUENCE %c%s%c.%c%s%c", q, dbt->database->name, q, q, dbt->table, q);
-  if (mysql_query(conn, query) || !(result = mysql_use_result(conn))) {
-    if (success_on_1146 && mysql_errno(conn) == 1146) {
-      g_warning("Error dumping schemas (%s.%s): %s", dbt->database->name, dbt->table,
-                mysql_error(conn));
-    } else {
-      g_critical("Error dumping schemas (%s.%s): %s", dbt->database->name, dbt->table,
-                 mysql_error(conn));
-      errors++;
-    }
-    g_free(query);
+  struct M_ROW *mr = m_store_result_row(conn, query, m_critical, m_warning, "Error dumping schemas (%s.%s)", dbt->database->name, dbt->table);
+  g_free(query);
+  if (!mr->res){
+    m_store_result_row_free(mr);
     return;
   }
   g_string_set_size(statement, 0);
 
   /* There should never be more than one row */
-  row = mysql_fetch_row(result);
-  if ( skip_definer && g_str_has_prefix(row[1],"CREATE")){
-    remove_definer_from_gchar(row[1]);
+  if ( skip_definer && g_str_has_prefix(mr->row[1],"CREATE")){
+    remove_definer_from_gchar(mr->row[1]);
   }
-  g_string_append(statement, row[1]);
+  g_string_append(statement, mr->row[1]);
   g_string_append(statement, ";\n");
   if (!write_data(outfile, statement)) {
     g_critical("Could not write schema for %s.%s", dbt->database->name, dbt->table);
     errors++;
   }
-  g_free(query);
-  if (result) {
-    mysql_free_result(result);
-    result = NULL;
-  }
+  m_store_result_row_free(mr);
 
   // Get current sequence position
   query = g_strdup_printf("SELECT next_not_cached_value FROM %c%s%c.%c%s%c", q, dbt->database->name, q, q, dbt->table, q);
-  if (mysql_query(conn, query) || !(result = mysql_use_result(conn))) {
-    if (success_on_1146 && mysql_errno(conn) == 1146) {
-      g_warning("Error dumping schemas (%s.%s): %s", dbt->database->name, dbt->table,
-                mysql_error(conn));
-    } else {
-      g_critical("Error dumping schemas (%s.%s): %s", dbt->database->name, dbt->table,
-                 mysql_error(conn));
-      errors++;
-    }
-    g_free(query);
-    return;
-  }
+  mr = m_store_result_row(conn, query, m_critical, m_warning, "Error dumping schemas (%s.%s)", dbt->database->name, dbt->table);
+  g_free(query);
+
   g_string_set_size(statement, 0);
   /* There should never be more than one row */
-  row = mysql_fetch_row(result);
-  g_string_printf(statement, "DO SETVAL(%c%s%c, %s, 0);\n", q, dbt->table, q, row[0]);
-  if (!write_data(outfile, statement)) {
-    g_critical("Could not write schema for %s.%s", dbt->database->name, dbt->table);
-    errors++;
+  if (mr->row){
+    g_string_printf(statement, "DO SETVAL(%c%s%c, %s, 0);\n", q, dbt->table, q, mr->row[0]);
+    if (!write_data(outfile, statement)) {
+      g_critical("Could not write schema for %s.%s", dbt->database->name, dbt->table);
+      errors++;
+    }
+    m_close(0, outfile, filename, 1, dbt);
+    // Table checksum should cover the basics, but doesn't checksum the current sequence position
+    if (checksum_filename)
+      write_checksum_into_file(conn, dbt->database, dbt->table, checksum_table_structure);
   }
-
-  g_free(query);
-  m_close(0, outfile, filename, 1, dbt);
+  m_store_result_row_free(mr);
   g_string_free(statement, TRUE);
-  if (result)
-    mysql_free_result(result);
-
-  // Table checksum should cover the basics, but doesn't checksum the current sequence position
-  if (checksum_filename)
-    write_checksum_into_file(conn, dbt->database, dbt->table, checksum_table_structure);
   return;
 }
 
@@ -657,10 +531,7 @@ static
 void write_routines_definition_into_file(MYSQL *conn, struct database *database, char *filename, gboolean checksum_filename) {
   int outfile;
   char *query = NULL;
-  MYSQL_RES *result = NULL;
-  MYSQL_RES *result2 = NULL;
   MYSQL_ROW row;
-  MYSQL_ROW row2;
   gchar **splited_st = NULL;
 
   outfile = m_open(&filename,"w");
@@ -672,7 +543,6 @@ void write_routines_definition_into_file(MYSQL *conn, struct database *database,
     return;
   }
 
-  const char q= identifier_quote_character;
   GString *statement = g_string_sized_new(statement_size);
   initialize_sql_statement(statement);
 
@@ -685,25 +555,18 @@ void write_routines_definition_into_file(MYSQL *conn, struct database *database,
   guint charcol=0,collcol=0;
   if (dump_routines) {
     g_assert(nroutines > 0);
+    struct M_ROW *mr=NULL;
     for (guint r= 0; r < nroutines; r++) {
       query= g_strdup_printf("SHOW %s STATUS WHERE CAST(Db AS BINARY) = '%s'", routine_type[r], database->escaped);
-      if (mysql_query(conn, query) || !(result = mysql_store_result(conn))) {
-        if (success_on_1146 && mysql_errno(conn) == 1146) {
-          g_warning("Error dumping %s from %s: %s", routine_type[r], database->name,
-                    mysql_error(conn));
-        } else {
-          g_critical("Error dumping %s from %s: %s", routine_type[r], database->name,
-                    mysql_error(conn));
-          errors++;
-        }
-        g_free(query);
+      MYSQL_RES *result = m_store_result_critical(conn, query,  "Error dumping %s from %s", routine_type[r], database->name);
+      g_free(query);
+      if (!result)
         return;
-      }
       determine_charset_and_coll_columns_from_show(result, &charcol, &collcol);
 
       while ((row= mysql_fetch_row(result))) {
         set_charset(statement, row[charcol], row[collcol]);
-        g_string_append_printf(statement, "DROP %s IF EXISTS %c%s%c;\n", routine_type[r], q, row[1], q);
+        g_string_append_printf(statement, "DROP %s IF EXISTS %c%s%c;\n", routine_type[r], identifier_quote_character, row[1], identifier_quote_character);
         if (!write_data(outfile, statement)) {
           g_critical("Could not write %s data for %s.%s", routine_type[r], database->name,
                     row[1]);
@@ -712,34 +575,29 @@ void write_routines_definition_into_file(MYSQL *conn, struct database *database,
           return;
         }
         g_string_set_size(statement, 0);
-        query= g_strdup_printf("SHOW CREATE %s %c%s%c.%c%s%c", routine_type[r], q, database->name, q,  q, row[1], q);
-        mysql_query(conn, query);
-        result2= mysql_store_result(conn);
-        row2= mysql_fetch_row(result2);
-        g_string_printf(statement, "%s", row2[2]);
-        if (skip_definer && g_str_has_prefix(statement->str, "CREATE")) {
-          remove_definer(statement);
-        }
-        splited_st= g_strsplit(statement->str, ";\n", 0);
-        g_string_printf(statement, "%s", g_strjoinv("; \n", splited_st));
-        g_string_append(statement, ";\n");
-        restore_charset(statement);
-        if (!write_data(outfile, statement)) {
-          g_critical("Could not write %s data for %s.%s", routine_type[r], database->name, row[1]);
-          errors++;
-          mysql_free_result(result);
-          if (result2)
-            mysql_free_result(result2);
-          return;
+        query= g_strdup_printf("SHOW CREATE %s %c%s%c.%c%s%c", routine_type[r], identifier_quote_character, database->name, identifier_quote_character,  identifier_quote_character, row[1], identifier_quote_character);
+        mr = m_store_result_single_row(conn, query, "Failed to execute SHOW CREATE %s %s.%s %s", routine_type[r], database->name, row[1], query);
+        g_free(query);
+        if (mr->row){
+          g_string_printf(statement, "%s", mr->row[2]);
+          if (skip_definer && g_str_has_prefix(statement->str, "CREATE")) {
+            remove_definer(statement);
+          }
+          splited_st= g_strsplit(statement->str, ";\n", 0);
+          g_string_printf(statement, "%s", g_strjoinv("; \n", splited_st));
+          g_string_append(statement, ";\n");
+          restore_charset(statement);
+          if (!write_data(outfile, statement)) {
+            g_critical("Could not write %s data for %s.%s", routine_type[r], database->name, mr->row[1]);
+            errors++;
+            continue;
+          }
         }
+        m_store_result_row_free(mr);
         g_string_set_size(statement, 0);
       }
 
       mysql_free_result(result);
-      if (result2) {
-        mysql_free_result(result2);
-        result2= NULL;
-      }
     } // for (guint r= 0; r < nroutines; r++)
 
     if (checksum_filename)
@@ -748,23 +606,15 @@ void write_routines_definition_into_file(MYSQL *conn, struct database *database,
 
   // get events
   if (dump_events) {
-    query = g_strdup_printf("SHOW EVENTS FROM %c%s%c", q, database->name, q);
-    if (mysql_query(conn, query) || !(result = mysql_store_result(conn))) {
-      if (success_on_1146 && mysql_errno(conn) == 1146) {
-        g_warning("Error dumping events from %s: %s", database->name,
-                  mysql_error(conn));
-      } else {
-        g_critical("Error dumping events from %s: %s", database->name,
-                   mysql_error(conn));
-        errors++;
-      }
-      g_free(query);
+    query = g_strdup_printf("SHOW EVENTS FROM %c%s%c", identifier_quote_character, database->name, identifier_quote_character);
+    MYSQL_RES *result = m_store_result_critical(conn, query,  "Error dumping events from %s", database->name);
+    g_free(query);
+    if (!result)
       return;
-    }
     determine_charset_and_coll_columns_from_show(result, &charcol, &collcol);
     while ((row = mysql_fetch_row(result))) {
       set_charset(statement, row[charcol], row[collcol]);
-      g_string_append_printf(statement, "DROP EVENT IF EXISTS %c%s%c;\n", q, row[1], q);
+      g_string_append_printf(statement, "DROP EVENT IF EXISTS %c%s%c;\n", identifier_quote_character, row[1], identifier_quote_character);
       if (!write_data(outfile, statement)) {
         g_critical("Could not write stored procedure data for %s.%s", database->name,
                    row[1]);
@@ -772,35 +622,33 @@ void write_routines_definition_into_file(MYSQL *conn, struct database *database,
         mysql_free_result(result);
         return;
       }
-      query = g_strdup_printf("SHOW CREATE EVENT %c%s%c.%c%s%c", q, database->name, q, q, row[1], q);
-      mysql_query(conn, query);
-      result2 = mysql_store_result(conn);
+      struct M_ROW *mr = m_store_result_row(conn, query=g_strdup_printf("SHOW CREATE EVENT %c%s%c.%c%s%c", identifier_quote_character, database->name, identifier_quote_character, identifier_quote_character, row[1], identifier_quote_character),
+          m_critical, m_warning, "Failed to execute SHOW CREATE EVENT %s.%s",database->name, row[1] );
+      g_free(query);
       // DROP EVENT IF EXISTS event_name
-      row2 = mysql_fetch_row(result2);
-      g_string_printf(statement, "%s", row2[3]);
-      if ( skip_definer && g_str_has_prefix(statement->str,"CREATE")){
-        remove_definer(statement);
-      }
-      splited_st = g_strsplit(statement->str, ";\n", 0);
-      g_string_printf(statement, "%s", g_strjoinv("; \n", splited_st));
-      g_string_append(statement, ";\n");
-      restore_charset(statement);
-      if (!write_data(outfile, statement)) {
-        g_critical("Could not write event data for %s.%s", database->name, row[1]);
-        errors++;
-        mysql_free_result(result);
-        if (result2)
-          mysql_free_result(result2);
-        return;
+      if (mr->row){
+        g_string_printf(statement, "%s", mr->row[3]);
+        if ( skip_definer && g_str_has_prefix(statement->str,"CREATE")){
+          remove_definer(statement);
+        }
+        splited_st = g_strsplit(statement->str, ";\n", 0);
+        g_string_printf(statement, "%s", g_strjoinv("; \n", splited_st));
+        g_string_append(statement, ";\n");
+        restore_charset(statement);
+        if (!write_data(outfile, statement)) {
+          g_critical("Could not write event data for %s.%s", database->name, mr->row[1]);
+          errors++;
+          m_store_result_row_free(mr);
+          goto clean;
+        }
       }
+      m_store_result_row_free(mr);
       g_string_set_size(statement, 0);
     }
     mysql_free_result(result);
-    if (result2)
-      mysql_free_result(result2);
   }
 
-  g_free(query);
+clean:
   m_close(0, outfile, filename, 1, NULL);
   g_string_free(statement, TRUE);
   g_strfreev(splited_st);
@@ -843,11 +691,11 @@ void free_table_checksum_job(struct table_checksum_job*tcj){
 }
 
 void do_JOB_CREATE_DATABASE(struct thread_data *td, struct job *job){
-  struct database_job * tj = (struct database_job *)job->job_data;
+  struct database_job * dj = (struct database_job *)job->job_data;
   g_message("Thread %d: dumping schema create for %s%s%s", td->thread_id,
-            identifier_quote_character_str, masquerade_filename?tj->database->filename:tj->database->name, identifier_quote_character_str);
-  write_schema_definition_into_file(td->thrconn, tj->database, tj->filename);
-  free_database_job(tj);
+            identifier_quote_character_str, masquerade_filename?dj->database->filename:dj->database->name, identifier_quote_character_str);
+  write_schema_definition_into_file(td->thrconn, dj->database, dj->filename);
+  free_database_job(dj);
   g_free(job);
 }
 
@@ -925,14 +773,14 @@ void do_JOB_CHECKSUM(struct thread_data *td, struct job *job){
   g_message("Thread %d: dumping checksum for %s%s%s.%s%s%s", td->thread_id,
                     identifier_quote_character_str, masquerade_filename?tj->dbt->database->filename:tj->dbt->database->name, identifier_quote_character_str,
                     identifier_quote_character_str, masquerade_filename?tj->dbt->table_filename:tj->dbt->table, identifier_quote_character_str);
-  if (use_savepoints && mysql_query(td->thrconn, "SAVEPOINT mydumper")) {
-    g_critical("Savepoint failed: %s", mysql_error(td->thrconn));
-  }
+  if (use_savepoints) 
+    m_query_critical(td->thrconn, "SAVEPOINT mydumper", "Savepoint failed");
+  
   tj->dbt->data_checksum=write_checksum_into_file(td->thrconn, tj->dbt->database, tj->dbt->table, checksum_table);
-  if (use_savepoints &&
-      mysql_query(td->thrconn, "ROLLBACK TO SAVEPOINT mydumper")) {
-    g_critical("Rollback to savepoint failed: %s", mysql_error(td->thrconn));
-  }
+
+  if (use_savepoints)
+      m_query_critical(td->thrconn, "ROLLBACK TO SAVEPOINT mydumper", "Rollback to savepoint failed");
+
   free_table_checksum_job(tj);
   g_free(job);
 }
