@@ -63,15 +63,22 @@ gboolean it_is_a_consistent_backup = FALSE;
 GHashTable *all_dbts=NULL;
 char * (*identifier_quote_character_protect)(char *r);
 struct configuration_per_table conf_per_table = {NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL};
+gboolean replica_stopped = FALSE;
+gboolean merge_dumpdir= FALSE;
+gboolean clear_dumpdir= FALSE;
+gboolean dirty_dumpdir= FALSE;
+
+// Program options used only on this file 
+extern guint ftwrl_max_wait_time;
+extern guint ftwrl_timeout_retries;
 
 // static variables
 static GMutex **pause_mutex_per_thread=NULL;
-static gboolean replica_stopped = FALSE;
 static guint pause_at=0;
 static guint resume_at=0;
 static gchar **db_items=NULL;
 static GRecMutex *ready_table_dump_mutex = NULL;
-
+static gboolean ftwrl_completed=FALSE;
 static
 void initialize_start_dump(){
   all_dbts=g_hash_table_new(g_str_hash, g_str_equal);
@@ -146,6 +153,63 @@ void *monitor_disk_space_thread (void *queue){
   return NULL;
 }
 
+// | Id  | User            | Host             | db   | Command | Time   | State                  | Info                  | Time_ms   | Rows_sent | Rows_examined |
+static
+void determine_columns_on_show_processlist( MYSQL_FIELD *fields, guint num_fields, int *id_col, int *user_col, int *command_col, int *time_col, int *info_col ){
+  /* Just in case PROCESSLIST output column order changes */
+  guint i;
+  for (i = 0; i < num_fields; i++) {
+    if (id_col && !strcasecmp(fields[i].name, "Id"))
+      *id_col = i;
+    else if (user_col && !strcasecmp(fields[i].name, "User"))
+      *user_col = i;
+    else if (command_col && !strcasecmp(fields[i].name, "Command"))
+      *command_col = i;
+    else if (time_col && !strcasecmp(fields[i].name, "Time"))
+      *time_col = i;
+    else if (info_col && !strcasecmp(fields[i].name, "Info"))
+      *info_col = i;
+  }
+  if ((     id_col && *id_col < 0) ||
+      (command_col && *command_col < 0) ||
+      (   time_col && *time_col < 0)){
+    m_critical("Error obtaining information from processlist");
+  }
+}
+
+void *monitor_ftwrl_thread (void *thread_id){
+  MYSQL *conn;
+  MYSQL_RES *res = NULL;
+  conn = mysql_init(NULL);
+  m_connect(conn);
+  gchar *query=NULL;
+  while (!ftwrl_completed){
+    sleep(ftwrl_max_wait_time);
+    res = m_store_result(conn,"SHOW PROCESSLIST", m_warning, "Could not check PROCESSLIST");
+    if (!res){
+       break;
+    } else {
+      MYSQL_ROW row;
+
+      /* Just in case PROCESSLIST output column order changes */
+      int id_col = -1, info_col=-1; 
+      determine_columns_on_show_processlist(mysql_fetch_fields(res), mysql_num_fields(res), &id_col, NULL, NULL, NULL, &info_col);
+      while ((row = mysql_fetch_row(res))) {
+        if ((atol(row[id_col]) == *((guint *)(thread_id)))){
+           if (!strcasecmp(FLUSH_TABLES_WITH_READ_LOCK, row[info_col]) || !strcasecmp(FLUSH_NO_WRITE_TO_BINLOG_TABLES, row[info_col])){
+             m_query_warning(conn, query = g_strdup_printf("KILL QUERY %lu", atol(row[id_col])), "Could not KILL slow query", NULL);
+             g_free(query);
+           }
+//            g_message("%s found. KILL %d",row[info_col], id_col);
+        }
+      }
+    }
+    mysql_free_result(res);
+  }
+  mysql_close(conn);
+  return NULL;
+}
+
 gboolean sig_triggered(void * user_data, int signal) {
   if (signal == SIGTERM){
     shutdown_triggered = TRUE;
@@ -247,13 +311,8 @@ MYSQL *create_connection() {
 static
 void detect_quote_character(MYSQL *conn)
 {
-  MYSQL_RES *res = NULL;
-  MYSQL_ROW row;
-
-  const char *query = "SELECT FIND_IN_SET('ANSI', @@SQL_MODE) OR FIND_IN_SET('ANSI_QUOTES', @@SQL_MODE)";
-
-  if (mysql_query(conn, query)){
-    g_warning("We were not able to determine ANSI mode: %s", mysql_error(conn));
+  MYSQL_RES *res = m_store_result(conn, "SELECT FIND_IN_SET('ANSI', @@SQL_MODE) OR FIND_IN_SET('ANSI_QUOTES', @@SQL_MODE)", m_warning, "We were not able to determine ANSI mode",NULL);
+  if (!res){
     identifier_quote_character= BACKTICK;
     identifier_quote_character_str= "`";
     fields_enclosed_by= "\"";
@@ -261,12 +320,8 @@ void detect_quote_character(MYSQL *conn)
 		return ;
   }
 
-  if (!(res = mysql_store_result(conn))){
-    g_warning("We were not able to determine ANSI mode");
-    return ;
-  }
-  row = mysql_fetch_row(res);
-  if (!strcmp(row[0], "0")) {
+  MYSQL_ROW row = mysql_fetch_row(res);
+  if (row && !strcmp(row[0], "0")) {
     identifier_quote_character= BACKTICK;
     identifier_quote_character_str= "`";
     fields_enclosed_by= "\"";
@@ -282,24 +337,19 @@ void detect_quote_character(MYSQL *conn)
 
 static
 void detect_sql_mode(MYSQL *conn){
-  MYSQL_RES *res = NULL;
-  MYSQL_ROW row;
-  GString *str;
+  struct M_ROW *mr = m_store_result_single_row(conn, "SELECT @@SQL_MODE", "Error getting SQL_MODE",NULL);
 
-  const char *query= "SELECT @@SQL_MODE";
-  if (mysql_query(conn, query)){
-    g_critical("Error getting SQL_MODE: %s", mysql_error(conn));
+  if (!mr->res || !mr->row){
+    m_store_result_row_free(mr);
+    return;
   }
 
-  if (!(res= mysql_store_result(conn))){
-    g_critical("Error getting SQL_MODE");
-  }
-  row= mysql_fetch_row(res);
-  str= g_string_new(NULL);
-  if (!g_strstr_len(row[0],-1, "NO_AUTO_VALUE_ON_ZERO"))
-    g_string_printf(str, "'NO_AUTO_VALUE_ON_ZERO,%s'", row[0]);
+  GString *str= g_string_new(NULL);
+
+  if (!g_strstr_len(mr->row[0],-1, "NO_AUTO_VALUE_ON_ZERO"))
+    g_string_printf(str, "'NO_AUTO_VALUE_ON_ZERO,%s'", mr->row[0]);
   else
-    g_string_printf(str, "'%s'", row[0]);
+    g_string_printf(str, "'%s'", mr->row[0]);
   g_string_replace(str, "NO_BACKSLASH_ESCAPES", "", 0);
   g_string_replace(str, ",,", ",", 0);
 
@@ -324,7 +374,7 @@ void detect_sql_mode(MYSQL *conn){
   g_string_replace(str, ",,", ",", 0);
   sql_mode= g_string_free(str, FALSE);
   g_assert(sql_mode);
-  mysql_free_result(res);
+  m_store_result_row_free(mr);
 }
 
 static
@@ -394,7 +444,6 @@ MYSQL *create_main_connection() {
 
 static
 void get_not_updated(MYSQL *conn, FILE *file) {
-  MYSQL_RES *res = NULL;
   MYSQL_ROW row;
 
   gchar *query =
@@ -402,20 +451,17 @@ void get_not_updated(MYSQL *conn, FILE *file) {
                       "information_schema.TABLES WHERE TABLE_TYPE = 'BASE "
                       "TABLE' AND UPDATE_TIME < NOW() - INTERVAL %d DAY",
                       updated_since);
-  if (mysql_query(conn, query)){
-    g_free(query);
-    return;
-  }
-
+  MYSQL_RES *res = m_store_result(conn, query, m_warning, "Updated since query failed", NULL);
   g_free(query);
 
-  if (!(res = mysql_store_result(conn)))
+  if (!res)
     return;
 
   while ((row = mysql_fetch_row(res))) {
     no_updated_tables = g_list_prepend(no_updated_tables, row[0]);
     fprintf(file, "%s\n", row[0]);
   }
+  mysql_free_result(res);
   no_updated_tables = g_list_reverse(no_updated_tables);
   fflush(file);
 }
@@ -423,201 +469,127 @@ void get_not_updated(MYSQL *conn, FILE *file) {
 static
 void long_query_wait(MYSQL *conn){
   char *p3=NULL;
-    while (TRUE) {
-      int longquery_count = 0;
-      if (mysql_query(conn, "SHOW PROCESSLIST")) {
-        g_warning("Could not check PROCESSLIST, no long query guard enabled: %s",
-                  mysql_error(conn));
-        break;
-      } else {
-        MYSQL_RES *res = mysql_store_result(conn);
-        MYSQL_ROW row;
+  while (TRUE) {
+    int longquery_count = 0;
+    MYSQL_RES *res = m_store_result(conn,"SHOW PROCESSLIST", m_warning, "Could not check PROCESSLIST, no long query guard enabled");
+    if (!res){
+       break;
+    } else {
+      MYSQL_ROW row;
 
-        /* Just in case PROCESSLIST output column order changes */
-        MYSQL_FIELD *fields = mysql_fetch_fields(res);
-        guint i;
-        int tcol = -1, ccol = -1, icol = -1, ucol = -1;
-        for (i = 0; i < mysql_num_fields(res); i++) {
-        if (!strcasecmp(fields[i].name, "Command"))
-            ccol = i;
-          else if (!strcasecmp(fields[i].name, "Time"))
-            tcol = i;
-          else if (!strcasecmp(fields[i].name, "Id"))
-            icol = i;
-          else if (!strcasecmp(fields[i].name, "User"))
-            ucol = i;
-        }
-        if ((tcol < 0) || (ccol < 0) || (icol < 0)) {
-          m_critical("Error obtaining information from processlist");
-        }
-        while ((row = mysql_fetch_row(res))) {
-          if (row[ccol] && strcmp(row[ccol], "Query"))
-            continue;
-          if (row[ucol] && !strcmp(row[ucol], "system user"))
-            continue;
-          if (row[tcol] && atoi(row[tcol]) > longquery) {
-            if (killqueries) {
-              if (mysql_query(conn,
-                              p3 = g_strdup_printf("KILL %lu", atol(row[icol])))) {
-                g_warning("Could not KILL slow query: %s", mysql_error(conn));
-                longquery_count++;
-              } else {
-                g_warning("Killed a query that was running for %ss", row[tcol]);
-              }
-              g_free(p3);
-            } else {
+      /* Just in case PROCESSLIST output column order changes */
+      int tcol = -1, ccol = -1, icol = -1, ucol = -1;
+      determine_columns_on_show_processlist(mysql_fetch_fields(res), mysql_num_fields(res), &icol, &ucol, &ccol, &tcol, NULL);
+      while ((row = mysql_fetch_row(res))) {
+        if (row[ccol] && strcmp(row[ccol], "Query"))
+          continue;
+        if (row[ucol] && !strcmp(row[ucol], "system user"))
+          continue;
+        if (row[tcol] && atoi(row[tcol]) > longquery) {
+          if (killqueries) {
+            if (m_query_warning(conn, p3 = g_strdup_printf("KILL %lu", atol(row[icol])), "Could not KILL slow query", NULL)){
               longquery_count++;
+            } else {
+              g_warning("Killed a query that was running for %ss", row[tcol]);
             }
+            g_free(p3);
+          } else {
+            longquery_count++;
           }
         }
-        mysql_free_result(res);
-        if (longquery_count == 0)
-          break;
-        else {
-          if (longquery_retries == 0) {
-            m_critical("There are queries in PROCESSLIST running longer than "
-                       "%us, aborting dump,\n\t"
-                       "use --long-query-guard to change the guard value, kill "
-                       "queries (--kill-long-queries) or use \n\tdifferent "
-                       "server for dump",
-                       longquery);
-          }
-          longquery_retries--;
-          g_warning("There are queries in PROCESSLIST running longer than "
-                         "%us, retrying in %u seconds (%u left).",
-                         longquery, longquery_retry_interval, longquery_retries);
-          sleep(longquery_retry_interval);
+      }
+      mysql_free_result(res);
+      if (longquery_count == 0)
+        break;
+      else {
+        if (longquery_retries == 0) {
+          m_critical("There are queries in PROCESSLIST running longer than "
+                     "%us, aborting dump,\n\t"
+                     "use --long-query-guard to change the guard value, kill "
+                     "queries (--kill-long-queries) or use \n\tdifferent "
+                     "server for dump",
+                     longquery);
         }
+        longquery_retries--;
+        g_warning("There are queries in PROCESSLIST running longer than "
+                       "%us, retrying in %u seconds (%u left).",
+                       longquery, longquery_retry_interval, longquery_retries);
+        sleep(longquery_retry_interval);
       }
     }
-}
-
-static
-int mysql_query_verbose(MYSQL *mysql, const char *q)
-{
-  int res= mysql_query(mysql, q);
-  if (!res)
-    g_message("%s: OK", q);
-  else
-    g_message("%s: %s (%d)", q, mysql_error(mysql), res);
-  return res;
+  }
 }
 
 static
 void send_backup_stage_on_block_commit(MYSQL *conn){
-/*
-  if (mysql_query(conn, "BACKUP STAGE START")) {
-    m_critical("Couldn't acquire BACKUP STAGE START: %s",
-               mysql_error(conn));
-    errors++;
-  }
-*/
-  if (mysql_query_verbose(conn, "BACKUP STAGE BLOCK_COMMIT")) {
-    m_critical("Couldn't acquire BACKUP STAGE BLOCK_COMMIT: %s",
-               mysql_error(conn));
-    errors++;
-  }
+  m_query_verbose(conn, "BACKUP STAGE BLOCK_COMMIT", m_critical, "Couldn't acquire BACKUP STAGE BLOCK_COMMIT", NULL);
 }
 
 static
 void send_mariadb_backup_locks(MYSQL *conn){
-  if (mysql_query_verbose(conn, "BACKUP STAGE START")) {
-    m_critical("Couldn't acquire BACKUP STAGE START: %s",
-               mysql_error(conn));
-    errors++;
-  }
-/*
-  if (mysql_query(conn, "BACKUP STAGE FLUSH")) {
-    m_critical("Couldn't acquire BACKUP STAGE FLUSH: %s",
-               mysql_error(conn));
-    errors++;
-  }
-*/
-  if (mysql_query_verbose(conn, "BACKUP STAGE BLOCK_DDL")) {
-    m_critical("Couldn't acquire BACKUP STAGE BLOCK_DDL: %s",
-               mysql_error(conn));
-    errors++;
-  }
-/*
-  if (mysql_query(conn, "BACKUP STAGE BLOCK_COMMIT")) {
-    m_critical("Couldn't acquire BACKUP STAGE BLOCK_COMMIT: %s",
-               mysql_error(conn));
-    errors++;
-  }
-*/
+  m_query_verbose(conn, "BACKUP STAGE START", m_critical, "Couldn't acquire BACKUP STAGE START", NULL);
+  m_query_verbose(conn, "BACKUP STAGE BLOCK_DDL", m_critical, "Couldn't acquire BACKUP STAGE BLOCK_DDL", NULL);
 }
 
 static
 void send_percona57_backup_locks(MYSQL *conn){
-  if (mysql_query_verbose(conn, "LOCK TABLES FOR BACKUP")) {
-    m_critical("Couldn't acquire LOCK TABLES FOR BACKUP, snapshots will "
-               "not be consistent: %s",
-               mysql_error(conn));
-    errors++;
-  }
-
-  if (mysql_query_verbose(conn, "LOCK BINLOG FOR BACKUP")) {
-    m_critical("Couldn't acquire LOCK BINLOG FOR BACKUP, snapshots will "
-               "not be consistent: %s",
-               mysql_error(conn));
-    errors++;
-  }
+  m_query_verbose(conn, "LOCK TABLES FOR BACKUP", m_critical, "Couldn't acquire LOCK TABLES FOR BACKUP, snapshots will not be consistent", NULL);
+  m_query_verbose(conn, "LOCK BINLOG FOR BACKUP", m_critical, "Couldn't acquire LOCK BINLOG FOR BACKUP, snapshots will not be consistent", NULL);
 }
 
 static
 void send_ddl_lock_instance_backup(MYSQL *conn){
-  if (mysql_query_verbose(conn, "LOCK INSTANCE FOR BACKUP")) {
-    m_critical("Couldn't acquire LOCK INSTANCE FOR BACKUP: %s",
-               mysql_error(conn));
-    errors++;
-  }
+  m_query_verbose(conn, "LOCK INSTANCE FOR BACKUP", m_critical, "Couldn't acquire LOCK INSTANCE FOR BACKUP", NULL);
 } 
 
 static
 void send_unlock_tables(MYSQL *conn){
-  mysql_query_verbose(conn, "UNLOCK TABLES");
+  m_query_verbose(conn, "UNLOCK TABLES", m_warning, "Failed to UNLOCK TABLES", NULL);
 }
 
 static
 void send_unlock_binlogs(MYSQL *conn){
-  mysql_query_verbose(conn, "UNLOCK BINLOG");
+  m_query_verbose(conn, "UNLOCK BINLOG", m_warning, "Failed to UNLOCK BINLOG", NULL);
 }
 
 static
 void send_ddl_unlock_instance_backup(MYSQL *conn){
-  mysql_query_verbose(conn, "UNLOCK INSTANCE");
+  m_query_verbose(conn, "UNLOCK INSTANCE", m_warning, "Failed to UNLOCK INSTANCE", NULL);
 }
 
 static
 void send_backup_stage_end(MYSQL *conn){
-  mysql_query_verbose(conn, "BACKUP STAGE END");
+  m_query_verbose(conn, "BACKUP STAGE END", m_warning, "Failed to BACKUP STAGE END", NULL);
 }
 
 static
 void send_flush_table_with_read_lock(MYSQL *conn){
-        if (mysql_query_verbose(conn, "FLUSH NO_WRITE_TO_BINLOG TABLES")) {
-          g_warning("Flush tables failed, we are continuing anyways: %s",
-                   mysql_error(conn));
-        }
-       if (mysql_query_verbose(conn, "FLUSH TABLES WITH READ LOCK")) {
-          g_critical("Couldn't acquire global lock, snapshots will not be "
-                   "consistent: %s",
-                   mysql_error(conn));
-          errors++;
-        }
-}
+  guint id=mysql_thread_id(conn);
+  m_thread_new("mon_ftwrl", monitor_ftwrl_thread, &id, "FTWRL monitor thread could not be created");
 
+  guint i=0;
+try_FLUSH_NO_WRITE_TO_BINLOG_TABLES:
+  i++;
+  if (( m_query_verbose(conn, FLUSH_NO_WRITE_TO_BINLOG_TABLES, m_warning, "Flush tables failed, we are continuing anyways")) && 
+      ( ftwrl_timeout_retries == 0 || (i < ftwrl_timeout_retries )))
+      goto try_FLUSH_NO_WRITE_TO_BINLOG_TABLES;
+
+try_FLUSH_TABLES_WITH_READ_LOCK:
+  if (( m_query_verbose(conn, FLUSH_TABLES_WITH_READ_LOCK, m_critical, "Couldn't acquire global lock, snapshots will not be consistent")) &&
+      ( ftwrl_timeout_retries == 0 || i < ftwrl_timeout_retries ))
+      goto try_FLUSH_TABLES_WITH_READ_LOCK;
+
+  ftwrl_completed=TRUE;
+}
 
 static
 void initialize_tidb_snapshot(MYSQL *conn){
   if (!tidb_snapshot){
     // Generate a @@tidb_snapshot to use for the worker threads since
     // the tidb-snapshot argument was not specified when starting mydumper
-    m_query(conn, show_binary_log_status, m_critical, "Couldn't generate @@tidb_snapshot");
-    MYSQL_RES *result = mysql_store_result(conn);
-    MYSQL_ROW row = mysql_fetch_row(result); /* There should never be more than one row */
-    tidb_snapshot = g_strdup(row[1]);
-    mysql_free_result(result);
+    struct M_ROW *mr = m_store_result_row(conn, show_binary_log_status, m_critical, m_warning, "Couldn't generate @@tidb_snapshot");
+    tidb_snapshot = g_strdup(mr->row[1]);
+    m_store_result_row_free(mr);
   }
   // Need to set the @@tidb_snapshot for the master thread
   set_tidb_snapshot(conn);
@@ -762,13 +734,8 @@ void send_lock_all_tables(MYSQL *conn){
     for (guint i = 0; tables[i] != NULL; i++) {
       dt = g_strsplit(tables[i], ".", 0);
       g_string_printf(query, "SHOW TABLES IN %s LIKE '%s'", dt[0], dt[1]);
-      if (mysql_query(conn, query->str)) {
-        g_error("Error showing tables in: %s - Could not execute query: %s", dt[0],
-                  mysql_error(conn));
-        errors++;
-        return;
-      }else {
-        res = mysql_store_result(conn);
+      res=m_store_result_critical(conn, query->str, "Error showing tables in: %s - Could not execute query", dt[0]);
+      if (res){
         while ((row = mysql_fetch_row(res))) {
           if (tables_skiplist_file && check_skiplist(dt[0], row[0]))
             continue;
@@ -805,12 +772,8 @@ void send_lock_all_tables(MYSQL *conn){
         "WHERE TABLE_TYPE ='BASE TABLE' AND TABLE_SCHEMA NOT IN "
         "('information_schema', 'performance_schema', 'data_dictionary')");
     }
-    if (mysql_query(conn, query->str)) {
-      g_critical("Couldn't get table list for lock all tables: %s",
-                 mysql_error(conn));
-      errors++;
-    } else {
-      res = mysql_store_result(conn);
+    res = m_store_result_critical(conn, query->str, "Couldn't get table list for lock all tables", NULL);
+    if (res){
       while ((row = mysql_fetch_row(res))) {
         // no need to check if the tb exists in the tables.
         if (tables_skiplist_file && check_skiplist(row[0], row[1]))
@@ -827,6 +790,7 @@ void send_lock_all_tables(MYSQL *conn){
   if (g_list_length(tables_lock) > 0) {
   // Try three times to get the lock, this is in case of tmp tables
   // disappearing
+    g_message("Initialing Lock All tables");
     while (g_list_length(tables_lock) > 0 && !success && retry < 4 ) {
       g_string_set_size(query,0);
       g_string_append(query, "LOCK TABLE ");
@@ -835,7 +799,7 @@ void send_lock_all_tables(MYSQL *conn){
       }
       g_strrstr(query->str,",")[0]=' ';
 
-      if (mysql_query(conn, query->str)) {
+      if (m_query_warning(conn, query->str, "Lock Table failed", NULL)) {
         gchar *failed_table = NULL;
         gchar **tmp_fail;
 
@@ -858,6 +822,7 @@ void send_lock_all_tables(MYSQL *conn){
     if (!success) {
       m_critical("Lock all tables fail: %s", mysql_error(conn));
     }
+    g_message("Lock All tables completed");
   }else{
     g_warning("No table found to lock");
 //    exit(EXIT_FAILURE);
@@ -866,7 +831,7 @@ void send_lock_all_tables(MYSQL *conn){
   g_free(query->str);
   g_list_free(tables_lock);
 }
-
+/*
 static
 void write_replica_info(MYSQL *conn, FILE *file) {
   MYSQL_RES *slave = NULL;
@@ -882,18 +847,20 @@ void write_replica_info(MYSQL *conn, FILE *file) {
   const char *gtid_title = NULL;
   guint i;
   guint isms = 0;
-  mysql_query(conn, "SELECT @@default_master_connection");
-  MYSQL_RES *rest = mysql_store_result(conn);
-  if (rest != NULL && mysql_num_rows(rest)) {
-    mysql_free_result(rest);
-    g_message("Multisource slave detected.");
-    isms = 1;
+  MYSQL_RES *rest=NULL;
+  if (get_product() == SERVER_TYPE_MARIADB ){
+    rest=m_store_result(conn, "SELECT @@default_master_connection", m_warning, "Variable @@default_master_connection not found", NULL);
+    if (rest != NULL && mysql_num_rows(rest)) {
+      mysql_free_result(rest);
+      g_message("Multisource slave detected.");
+      isms = 1;
+    }
   }
 
   if (isms)
-    m_query(conn, show_all_replicas_status , m_critical, "Error executing %s", show_all_replicas_status);
+    m_query_critical(conn, show_all_replicas_status, "Error executing %s", show_all_replicas_status);
   else
-    m_query(conn, show_replica_status, m_critical, "Error executing %s", show_replica_status);
+    m_query_critical(conn, show_replica_status, "Error executing %s", show_replica_status);
 
   guint slave_count=0;
   slave = mysql_store_result(conn);
@@ -903,19 +870,15 @@ void write_replica_info(MYSQL *conn, FILE *file) {
   }
   mysql_free_result(slave);
   g_message("Stopping replica");
-  replica_stopped=!mysql_query(conn, stop_replica_sql_thread);
-  if (!replica_stopped){
-    g_warning("Not able to stop replica: %s", mysql_error(conn));
-  }
+  replica_stopped=!m_query_warning(conn, stop_replica_sql_thread, "Not able to stop replica",NULL);
   if (source_control_command==AWS){
     discard_mysql_output(conn);
   } 
 
-
   if (isms)  
-    m_query(conn, show_all_replicas_status, m_critical, "Error executing %s", show_all_replicas_status);
+    m_query_critical(conn, show_all_replicas_status, "Error executing %s", show_all_replicas_status);
   else
-    m_query(conn, show_replica_status, m_critical, "Error executing %s", show_replica_status);
+    m_query_critical(conn, show_replica_status, "Error executing %s", show_replica_status);
 
   slave = mysql_store_result(conn);
 
@@ -970,7 +933,42 @@ cleanup:
   if (slave)
     mysql_free_result(slave);
 }
+*/
 
+guint isms = 0;
+static
+void m_stop_replica(MYSQL *conn) {
+  MYSQL_RES *slave = NULL;
+  MYSQL_RES *rest=NULL;
+  if (get_product() == SERVER_TYPE_MARIADB ){
+    rest=m_store_result(conn, "SELECT @@default_master_connection", m_warning, "Variable @@default_master_connection not found", NULL);
+    if (rest != NULL && mysql_num_rows(rest)) {
+      mysql_free_result(rest);
+      g_message("Multisource slave detected.");
+      isms = 1;
+    }
+  }
+
+  if (isms)
+    m_query_critical(conn, show_all_replicas_status, "Error executing %s", show_all_replicas_status);
+  else
+    m_query_critical(conn, show_replica_status, "Error executing %s", show_replica_status);
+
+  slave = mysql_store_result(conn);
+
+  if (!slave || mysql_num_rows(slave) == 0){
+    goto cleanup;
+  }
+  g_message("Stopping replica");
+  replica_stopped=!m_query_warning(conn, stop_replica_sql_thread, "Not able to stop replica",NULL);
+  if (source_control_command==AWS){
+    discard_mysql_output(conn);
+  }
+
+cleanup:
+  if (slave)
+    mysql_free_result(slave);
+}
 
 // Here is where the backup process start
 
@@ -994,9 +992,10 @@ void start_dump() {
   // Initializing process
   if (clear_dumpdir)
     clear_dump_directory(dump_directory);
-  else if (!dirty_dumpdir && !is_empty_dir(dump_directory)) {
-    g_error("Directory is not empty (use --clear or --dirty): %s\n", dump_directory);
+  else if (!(dirty_dumpdir || merge_dumpdir) && !is_empty_dir(dump_directory)) {
+    g_error("Directory is not empty (use --clear, --dirty or --merge): %s\n", dump_directory);
   }
+
   check_num_threads();
   g_message("Using %u dumper threads", num_threads);
   initialize_start_dump();
@@ -1025,9 +1024,13 @@ void start_dump() {
 
   if (disk_limits!=NULL){
     conf.pause_resume = g_async_queue_new();
-    disk_check_thread = m_thread_new("monitor",monitor_disk_space_thread, conf.pause_resume, "Monitor thread could not be created");
+    disk_check_thread = m_thread_new("mon_disk",monitor_disk_space_thread, conf.pause_resume, "Monitor thread could not be created");
   }
 
+//  GThread *throttling_thread = 
+  if (throttle_variable)
+    m_thread_new("mon_thro",monitor_throttling_thread, NULL, "Monitor throttling thread could not be created");
+
   // signal_thread is disable if daemon mode
   if (!daemon_mode)
     sthread = m_thread_new("signal", signal_thread, &conf, "Signal thread could not be created");
@@ -1042,7 +1045,11 @@ void start_dump() {
     metadata_partial_filename= g_strdup_printf("%s/metadata.partial", dump_directory);
   metadata_filename = g_strdup_printf("%s/metadata", dump_directory);
 
-  mdfile = g_fopen(metadata_partial_filename, "w");
+  if (merge_dumpdir)
+   if (g_rename(metadata_filename, metadata_partial_filename))
+     m_critical("We were not able to rename metadata (%s) file to %s",metadata_filename, metadata_partial_filename);
+
+  mdfile = g_fopen(metadata_partial_filename, "a");
   if (!mdfile) {
     m_critical("Couldn't create metadata file %s (%s)", metadata_partial_filename, strerror(errno));
   }
@@ -1073,14 +1080,14 @@ void start_dump() {
   g_free(datetimestr);
 
   /* Write dump config into beginning of metadata, stream this first */
-  {
-    g_assert(identifier_quote_character == BACKTICK || identifier_quote_character == DOUBLE_QUOTE);
-    const char *qc= identifier_quote_character == BACKTICK ? "BACKTICK" : "DOUBLE_QUOTE";
-    fprintf(mdfile, "[config]\nquote_character = %s\n", qc);
-    fprintf(mdfile, "\n[myloader_session_variables]");
-    fprintf(mdfile, "\nSQL_MODE=%s /*!40101\n\n", sql_mode);
-    fflush(mdfile);
-  }
+  g_assert(identifier_quote_character == BACKTICK || identifier_quote_character == DOUBLE_QUOTE);
+  const char *qc= identifier_quote_character == BACKTICK ? "BACKTICK" : "DOUBLE_QUOTE";
+  fprintf(mdfile, "[config]\nquote-character = %s\n", qc);
+  if (load_data || csv )
+    fprintf(mdfile, "local-infile = 1\n");
+  fprintf(mdfile, "\n[myloader_session_variables]");
+  fprintf(mdfile, "\nSQL_MODE=%s /*!40101\n\n", sql_mode);
+  fflush(mdfile);
 
   // Initilizing stream backup
   if (stream){
@@ -1103,7 +1110,7 @@ void start_dump() {
   // Write replica information
   if (get_product() != SERVER_TYPE_TIDB) {
     if (source_data >=0 )
-      write_replica_info(conn, mdfile);
+      m_stop_replica(conn);
   }
 
   // Determine the locking mechanisim that is going to be used
@@ -1145,9 +1152,7 @@ void start_dump() {
   // TODO: this should be deleted on future releases. 
   server_version= mysql_get_server_version(conn);
   if (server_version < 40108) {
-    mysql_query(
-        conn,
-        "CREATE TABLE IF NOT EXISTS mysql.mydumperdummy (a INT) ENGINE=INNODB");
+    m_query_warning(conn, "CREATE TABLE IF NOT EXISTS mysql.mydumperdummy (a INT) ENGINE=INNODB", "Not able to create dummy table for InnoDB", NULL);
     need_dummy_read = 1;
   }
   // TODO: MySQL also supports PACKAGE (Percona?)
@@ -1155,30 +1160,26 @@ void start_dump() {
     nroutines= 2;
 
   // tokudb do not support consistent snapshot
-  mysql_query(conn, "SELECT @@tokudb_version");
-  MYSQL_RES *rest = mysql_store_result(conn);
-  if (rest != NULL && mysql_num_rows(rest)) {
+  MYSQL_RES *rest = m_store_result(conn, "SELECT @@tokudb_version", m_message, "@@tokudb_version not found", NULL);
+  if (rest){
+    if (mysql_num_rows(rest)) {
+      mysql_free_result(rest);
+      g_message("TokuDB detected, creating dummy table for CS");
+      m_query_warning(conn, "CREATE TABLE IF NOT EXISTS mysql.tokudbdummy (a INT) ENGINE=TokuDB", "Not able to create dummy table for TokuDB", NULL);
+      need_dummy_toku_read = 1;
+    }
     mysql_free_result(rest);
-    g_message("TokuDB detected, creating dummy table for CS");
-    mysql_query(
-        conn,
-        "CREATE TABLE IF NOT EXISTS mysql.tokudbdummy (a INT) ENGINE=TokuDB");
-    need_dummy_toku_read = 1;
   }
 
   if (need_dummy_read) {
-    mysql_query(conn,
-                "SELECT /*!40001 SQL_NO_CACHE */ * FROM mysql.mydumperdummy");
-    MYSQL_RES *res = mysql_store_result(conn);
-    if (res)
-      mysql_free_result(res);
+    rest = m_store_result(conn, "SELECT /*!40001 SQL_NO_CACHE */ * FROM mysql.mydumperdummy", m_warning, "Select on mysql.mydumperdummy has failed", NULL);
+    if (rest)
+      mysql_free_result(rest);
   }
   if (need_dummy_toku_read) {
-    mysql_query(conn,
-                "SELECT /*!40001 SQL_NO_CACHE */ * FROM mysql.tokudbdummy");
-    MYSQL_RES *res = mysql_store_result(conn);
-    if (res)
-      mysql_free_result(res);
+    rest = m_store_result(conn, "SELECT /*!40001 SQL_NO_CACHE */ * FROM mysql.tokudbdummy", m_warning, "Select on mysql.tokudbdummy has failed", NULL);
+    if (rest)
+      mysql_free_result(rest);
   }
 
   // Initilizing the configuration
@@ -1259,18 +1260,17 @@ void start_dump() {
   if (trx_tables) {
     // Releasing locks as user instructed that all tables are transactional
     g_message("Transactions started, unlocking tables");
-    if (release_global_lock_function)
-      release_global_lock_function(conn);
     if (release_binlog_function != NULL){
       g_async_queue_pop(conf.binlog_ready);
       g_message("Releasing binlog lock");
       release_binlog_function(second_conn);
     }
-    if (replica_stopped){
+    if (release_global_lock_function)
+      release_global_lock_function(conn);
+    if (is_mysql_like() && g_async_queue_pop(conf.binlog_ready) && replica_stopped){
       g_message("Starting replica");
-      if (mysql_query(conn, start_replica_sql_thread)){
-        g_warning("Not able to start replica: %s", mysql_error(conn));
-      }
+      m_query_warning(conn, start_replica_sql_thread, "Not able to start replica", NULL);
+
       if (source_control_command==AWS){
         discard_mysql_output(conn);
       }
@@ -1324,23 +1324,22 @@ void start_dump() {
     for (n = 0; n < num_threads; n++) {
       g_async_queue_pop(conf.unlock_tables);
     }
-    g_message("Non-InnoDB dump complete, releasing global locks");
-    if (release_global_lock_function)
-      release_global_lock_function(conn);
-    g_message("Global locks released");
     if (release_binlog_function != NULL){
       g_async_queue_pop(conf.binlog_ready);
       g_message("Releasing binlog lock");
       release_binlog_function(second_conn);
     }
+    g_message("Non-InnoDB dump complete, releasing global locks");
+    if (release_global_lock_function)
+      release_global_lock_function(conn);
+    g_message("Global locks released");
   }
 
   // At this point, we can start the replica if it was stopped
-  if (replica_stopped){
+  if (is_mysql_like() && g_async_queue_pop(conf.binlog_ready) && replica_stopped){
     g_message("Starting replica");
-    if (mysql_query(conn, start_replica_sql_thread)){
-      g_warning("Not able to start replica: %s", mysql_error(conn));
-    }
+    m_query_warning(conn, start_replica_sql_thread, "Not able to start replica", NULL);
+
     if (source_control_command==AWS){
       discard_mysql_output(conn);
     }
@@ -1427,7 +1426,8 @@ void start_dump() {
   if (updated_since > 0)
     fclose(nufile);
 
-  g_rename(metadata_partial_filename, metadata_filename);
+  if (g_rename(metadata_partial_filename, metadata_filename))
+    m_critical("We were not able to rename metadata file");
 
   if (stream) {
     stream_queue_push(NULL, g_strdup(metadata_filename));
