@@ -230,11 +230,12 @@ struct chunk_step_item{
   enum chunk_type chunk_type;
   struct chunk_step_item *next;
   struct chunk_functions chunk_functions;
+  gboolean multicolumn;
   GString *where;
   gboolean include_null;
   GString *prefix;
   gchar *field;
-  guint64 number;
+  guint64 part;
   guint deep;
   guint position;
   GMutex *mutex;
@@ -254,7 +255,7 @@ struct table_job_file{
 // second number : when load data is used
 struct table_job {
   char *partition;
-  guint64 nchunk;
+  guint64 part;
   guint sub_part;
   GString *where;
 //  union chunk_step *chunk_step;  
@@ -272,6 +273,7 @@ struct table_job {
   int child_process;
   int char_chunk_part;
   struct thread_data *td;
+  guint64 num_rows_of_last_run;
 };
 
 struct dump_table_job{
