@@ -38,8 +38,6 @@
 /* Program options */
 gboolean order_by_primary_key = FALSE;
 gboolean use_savepoints = FALSE;
-gboolean clear_dumpdir= FALSE;
-gboolean dirty_dumpdir= FALSE;
 gchar *ignore_engines_str = NULL;
 int skip_tz = 0;
 gboolean dump_events = FALSE;
@@ -66,6 +64,9 @@ struct MList  *non_transactional_table = NULL;
 guint64 min_chunk_step_size = 0;
 guint64 starting_chunk_step_size = 0;
 guint64 max_chunk_step_size = 0;
+gchar *exec_per_thread = NULL;
+const gchar *exec_per_thread_extension = NULL;
+gchar **exec_per_thread_cmd=NULL;
 
 // Static
 static GMutex *init_mutex = NULL;
@@ -130,31 +131,37 @@ void initialize_working_thread(){
 
 // TODO: We need to cleanup this
 
-  if (compress_method==NULL && exec_per_thread==NULL && exec_per_thread_extension == NULL) {
+  if ((exec_per_thread_extension==NULL) && (exec_per_thread != NULL))
+    m_critical("--exec-per-thread-extension needs to be set when --exec-per-thread (%s) is used", exec_per_thread);
+  if ((exec_per_thread_extension!=NULL) && (exec_per_thread == NULL))
+    m_critical("--exec-per-thread needs to be set when --exec-per-thread-extension (%s) is used", exec_per_thread_extension);
+
+  if (compress_method==NULL && exec_per_thread==NULL) {
     exec_per_thread_extension=EMPTY_STRING;
     initialize_file_handler(FALSE);
   }else{
-    if (compress_method!=NULL && (exec_per_thread!=NULL || exec_per_thread_extension!=NULL)){
+    if (compress_method!=NULL && exec_per_thread!=NULL )
       m_critical("--compression and --exec-per-thread are not comptatible");
-    }
-    gchar *cmd=NULL;
-    if ( g_strcmp0(compress_method,GZIP)==0){
-      if ((cmd=get_gzip_cmd()) == NULL){
-        g_error("gzip command not found on any static location, use --exec-per-thread for non default locations");
+    
+    if (compress_method){
+      if ( g_ascii_strcasecmp(compress_method,GZIP)==0){
+        exec_per_thread=g_strdup_printf("%s -c", GZIP);
+        exec_per_thread_extension=GZIP_EXTENSION;
+      }else if (g_ascii_strcasecmp(compress_method,ZSTD)==0){
+        exec_per_thread=g_strdup_printf("%s -c", ZSTD);
+        exec_per_thread_extension=ZSTD_EXTENSION;
       }
-      exec_per_thread=g_strdup_printf("%s -c", cmd);
-      exec_per_thread_extension=GZIP_EXTENSION;
-    }else 
-    if ( g_strcmp0(compress_method,ZSTD)==0){
-      if ( (cmd=get_zstd_cmd()) == NULL ){
-        g_error("zstd command not found on any static location, use --exec-per-thread for non default locations");
-      }
-      exec_per_thread=g_strdup_printf("%s -c", cmd);
-      exec_per_thread_extension=ZSTD_EXTENSION;
     }
     initialize_file_handler(TRUE);
+
+    exec_per_thread_cmd=g_strsplit(exec_per_thread, " ", 0);
+    gchar *tmpcmd=g_find_program_in_path(exec_per_thread_cmd[0]);
+    if (!tmpcmd)
+      m_critical("%s was not found in PATH, use --exec-per-thread for non default locations",exec_per_thread_cmd[0]);
+    exec_per_thread_cmd[0]=tmpcmd;
   }
 
+
   initialize_jobs();
   initialize_chunk();
 
@@ -243,9 +250,8 @@ void get_primary_key(MYSQL *conn, struct db_table * dbt, struct configuration *c
   //    * configuration too 
   gchar *query = g_strdup_printf("SHOW INDEX FROM %s%s%s.%s%s%s",
                         identifier_quote_character_str, dbt->database->name, identifier_quote_character_str, identifier_quote_character_str, dbt->table, identifier_quote_character_str);
-  mysql_query(conn, query);
+  indexes = m_store_result(conn, query, m_warning, "Failed to execute SHOW INDEX over %s", dbt->database->name);
   g_free(query);
-  indexes = mysql_store_result(conn);
 
   if (indexes){
     while ((row = mysql_fetch_row(indexes))) {
@@ -299,11 +305,7 @@ void thd_JOB_DUMP_ALL_DATABASES( struct thread_data *td, struct job *job){
   // TODO: This should be in a job as needs to be done by a thread.
   MYSQL_RES *databases;
   MYSQL_ROW row;
-  if (mysql_query(td->thrconn, "SHOW DATABASES") ||
-      !(databases = mysql_store_result(td->thrconn))) {
-    m_critical("Unable to list databases: %s", mysql_error(td->thrconn));
-    return;
-  }
+  databases=m_store_result(td->thrconn, "SHOW DATABASES", m_critical, "Unable to list databases", NULL);
 
   while ((row = mysql_fetch_row(databases))) {
     if (!strcasecmp(row[0], "information_schema") ||
@@ -353,15 +355,13 @@ void get_table_info_to_process_from_list(MYSQL *conn, struct configuration *conf
     dt = g_strsplit(table_list[x], ".", 0);
 
     query= g_strdup_printf("SHOW TABLE STATUS FROM %s%s%s LIKE '%s'", identifier_quote_character_str, dt[0], identifier_quote_character_str, dt[1]);
+    MYSQL_RES *result = m_store_result(conn, query, m_critical, "Error showing table status on: %s - Could not execute query", dt[0]);
+    g_free(query);
 
-    if (mysql_query(conn, (query))) {
-      g_critical("Error showing table status on: %s - Could not execute query: %s", dt[0],
-                 mysql_error(conn));
-      errors++;
+    if (!result) {
       return;
     }
 
-    MYSQL_RES *result = mysql_store_result(conn);
     guint ecol = -1, ccol = -1, collcol = -1, rowscol = 0;
     determine_show_table_status_columns(result, &ecol, &ccol, &collcol, &rowscol);
     struct database * database=NULL;
@@ -377,12 +377,6 @@ void get_table_info_to_process_from_list(MYSQL *conn, struct configuration *conf
       }
     }
 
-    if (!result) {
-      g_critical("Could not list tables for %s: %s", database->name, mysql_error(conn));
-      errors++;
-      return;
-    }
-
     MYSQL_ROW row;
     while ((row = mysql_fetch_row(result))) {
 
@@ -419,7 +413,6 @@ void get_table_info_to_process_from_list(MYSQL *conn, struct configuration *conf
     g_strfreev(dt);
   }
 
-  g_free(query);
   if (g_atomic_int_dec_and_test(&database_counter)) {
     g_async_queue_push(conf->db_ready,GINT_TO_POINTER(1));
 //    g_rec_mutex_unlock(ready_database_dump_mutex);
@@ -457,40 +450,28 @@ void m_async_queue_push_conservative(GAsyncQueue *queue, struct job *element){
   g_async_queue_push(queue, element);
 }
 
-
 void thd_JOB_DUMP(struct thread_data *td, struct job *job){
   struct table_job *tj = (struct table_job *)job->job_data;
 
   if (use_savepoints){
     if (td->table_name!=NULL){
       if (tj->dbt->table != td->table_name){
-        if ( mysql_query(td->thrconn, "ROLLBACK TO SAVEPOINT mydumper")) {
-          g_critical("Rollback to savepoint failed: %s", mysql_error(td->thrconn));
-        }
-        if (mysql_query(td->thrconn, "SAVEPOINT mydumper")) {
-          g_critical("Savepoint failed: %s", mysql_error(td->thrconn));
-        }
+        m_query_critical(td->thrconn, "ROLLBACK TO SAVEPOINT mydumper", "Rollback to savepoint failed", NULL);
+        m_query_critical(td->thrconn, "SAVEPOINT mydumper","Savepoint failed", NULL);
         td->table_name = tj->dbt->table;
       }
     }else{
-      if (mysql_query(td->thrconn, "SAVEPOINT mydumper")) {
-        g_critical("Savepoint failed: %s", mysql_error(td->thrconn));
-      }
+      m_query_critical(td->thrconn, "SAVEPOINT mydumper", "Savepoint failed", NULL);
       td->table_name = tj->dbt->table;
     }
   }
   tj->td=td;
 
-//  g_debug("chunk_type: %d %p", tj->dbt->chunk_type, tj->dbt->chunk_functions.process);
   tj->chunk_step_item->chunk_functions.process(tj, tj->chunk_step_item);
   g_mutex_lock(tj->dbt->chunks_mutex);
   tj->dbt->current_threads_running--;
   g_mutex_unlock(tj->dbt->chunks_mutex);
 
-/*  if (use_savepoints &&
-      mysql_query(td->thrconn, "ROLLBACK TO SAVEPOINT mydumper")) {
-    g_critical("Rollback to savepoint failed: %s", mysql_error(td->thrconn));
-  }*/
   free_table_job(tj);
   g_free(job);
 }
@@ -502,10 +483,10 @@ void initialize_thread(struct thread_data *td){
 }
 
 void initialize_consistent_snapshot(struct thread_data *td){
-  if ( sync_wait != -1 && mysql_query(td->thrconn, g_strdup_printf("SET SESSION WSREP_SYNC_WAIT = %d",sync_wait))){
-    m_critical("Failed to set wsrep_sync_wait for the thread: %s",
-               mysql_error(td->thrconn));
-  }
+  gchar *query=NULL;
+  if ( sync_wait != -1)
+    m_query_critical(td->thrconn, query=g_strdup_printf("SET SESSION WSREP_SYNC_WAIT = %d",sync_wait), "Failed to set wsrep_sync_wait for the thread",NULL);
+  g_free(query);
   set_transaction_isolation_level_repeatable_read(td->thrconn);
   guint start_transaction_retry=0;
   gboolean cont = FALSE;
@@ -515,26 +496,17 @@ void initialize_consistent_snapshot(struct thread_data *td){
 //    Uncommenting the sleep will cause inconsitent scenarios always, which is useful for debugging 
 //      sleep(td->thread_id);
       g_debug("Thread %d: Start transaction #%d", td->thread_id, start_transaction_retry);
-      if (mysql_query(td->thrconn,
-                    "START TRANSACTION /*!40108 WITH CONSISTENT SNAPSHOT */")) {
-        m_critical("Failed to start consistent snapshot: %s", mysql_error(td->thrconn));
-      }
-      if (mysql_query(td->thrconn,
-                    "SHOW STATUS LIKE 'binlog_snapshot_gtid_executed'")) {
-        m_critical("Failed to get binlog_snapshot_gtid_executed: %s", mysql_error(td->thrconn));
-      }else{
-        MYSQL_RES *res = mysql_store_result(td->thrconn);
-        if (res){
-          MYSQL_ROW row = mysql_fetch_row(res);
-          if (row!=NULL)
-            td->binlog_snapshot_gtid_executed=g_strdup(row[1]);
-          else
-            m_critical("Failed to get content of binlog_snapshot_gtid_executed");
-          mysql_free_result(res);
-        }else{
-          m_critical("Failed to get content of binlog_snapshot_gtid_executed: %s", mysql_error(td->thrconn));
-        }
-      }
+      m_query_critical(td->thrconn,"START TRANSACTION /*!40108 WITH CONSISTENT SNAPSHOT */", "Failed to start consistent snapshot", NULL);
+      MYSQL_RES *res = m_store_result_critical (td->thrconn, "SHOW STATUS LIKE 'binlog_snapshot_gtid_executed'", "Failed to get binlog_snapshot_gtid_executed", NULL);
+      if (res){
+        MYSQL_ROW row = mysql_fetch_row(res);
+        if (row!=NULL)
+          td->binlog_snapshot_gtid_executed=g_strdup(row[1]);
+        else
+          m_critical("Failed to get content of binlog_snapshot_gtid_executed");
+        mysql_free_result(res);
+      }else
+        m_critical("Failed to get content of binlog_snapshot_gtid_executed: %s", mysql_error(td->thrconn));
       start_transaction_retry++;
       g_async_queue_push(td->conf->gtid_pos_checked, GINT_TO_POINTER(1));
       cont=GPOINTER_TO_INT(g_async_queue_pop(td->conf->are_all_threads_in_same_pos)) == 1;
@@ -546,11 +518,8 @@ void initialize_consistent_snapshot(struct thread_data *td){
     }else{
       m_critical("We were not able to sync all threads. We unsuccessfully tried %d times. Reducing the amount of threads might help.", MAX_START_TRANSACTION_RETRIES);
     }
-  }else{
-    if (mysql_query(td->thrconn,
-                  "START TRANSACTION /*!40108 WITH CONSISTENT SNAPSHOT */"))
-      m_critical("Failed to start consistent snapshot: %s", mysql_error(td->thrconn));
-  }
+  }else
+    m_query_critical(td->thrconn,"START TRANSACTION /*!40108 WITH CONSISTENT SNAPSHOT */", "Failed to start consistent snapshot", NULL);
 }
 
 static
@@ -566,16 +535,12 @@ void check_connection_status(struct thread_data *td){
   /* Unfortunately version before 4.1.8 did not support consistent snapshot
    * transaction starts, so we cheat */
   if (need_dummy_read) {
-    mysql_query(td->thrconn,
-                "SELECT /*!40001 SQL_NO_CACHE */ * FROM mysql.mydumperdummy");
-    MYSQL_RES *res = mysql_store_result(td->thrconn);
+    MYSQL_RES *res = m_store_result(td->thrconn, "SELECT /*!40001 SQL_NO_CACHE */ * FROM mysql.mydumperdummy", m_warning, "Failed to select on mysql.mydumperdummy", NULL);
     if (res)
       mysql_free_result(res);
   }
   if (need_dummy_toku_read) {
-    mysql_query(td->thrconn,
-                "SELECT /*!40001 SQL_NO_CACHE */ * FROM mysql.tokudbdummy");
-    MYSQL_RES *res = mysql_store_result(td->thrconn);
+    MYSQL_RES *res = m_store_result(td->thrconn, "SELECT /*!40001 SQL_NO_CACHE */ * FROM mysql.tokudbdummy", m_warning, "Failed to select on mysql.tokudbdummy", NULL);
     if (res)
       mysql_free_result(res);
   }
@@ -584,41 +549,36 @@ void check_connection_status(struct thread_data *td){
 
 /* Write some stuff we know about snapshot, before it changes */
 void write_snapshot_info(MYSQL *conn, FILE *file) {
-  MYSQL_RES *master = NULL, *mdb = NULL;
-  MYSQL_ROW row;
-
   char *masterlog = NULL;
   char *masterpos = NULL;
   char *mastergtid = NULL;
 
   
-  if (mysql_query(conn, show_binary_log_status))
-    m_warning("Couldn't get master position: %s", mysql_error(conn));
-    
-  master = mysql_store_result(conn);
-  if (master && (row = mysql_fetch_row(master))) {
-    masterlog = row[0];
-    masterpos = row[1];
+  struct M_ROW *mr = m_store_result_row(conn, show_binary_log_status, m_warning, m_message, "Couldn't get master position", NULL);
+  if ( mr->row ) {
+    masterlog = g_strdup(mr->row[0]);
+    masterpos = g_strdup(mr->row[1]);
     /* Oracle/Percona GTID */
-    if (mysql_num_fields(master) == 5) {
-      mastergtid = remove_new_line(row[4]);
+    if (mysql_num_fields(mr->res) == 5) {
+      mastergtid = g_strdup(remove_new_line(mr->row[4]));
     } else {
       /* Let's try with MariaDB 10.x */
       /* Use gtid_binlog_pos due to issue with gtid_current_pos with galera
  *        * cluster, gtid_binlog_pos works as well with normal mariadb server
  *               * https://jira.mariadb.org/browse/MDEV-10279 */
-      mysql_query(conn, "SELECT @@gtid_binlog_pos");
-      mdb = mysql_store_result(conn);
-      if (mdb && (row = mysql_fetch_row(mdb))) {
-        mastergtid = remove_new_line(row[0]);
+      m_store_result_row_free(mr);
+      mr = m_store_result_row(conn, "SELECT @@gtid_binlog_pos", NULL, NULL, "Failed to get @@gtid_binlog_pos", NULL);
+      if (mr->row){
+        mastergtid = g_strdup(remove_new_line(mr->row[0]));
       }
     }
   }
+  m_store_result_row_free(mr);
 
   if (masterlog) {
     fprintf(file, "[source]\n# Channel_Name = '' # It can be use to setup replication FOR CHANNEL\n");
     if (source_data > 0){
-      fprintf(file, "#SOURCE_HOST = \"%s\"\n#SOURCE_PORT = \n#SOURCE_USER = \"\"\n#SOURCE_PASSWORD = \"\"\n", hostname);
+      fprintf(file, "#SOURCE_HOST = \"%s\"\n#SOURCE_PORT = \n#SOURCE_USER = \"\"\n#SOURCE_PASSWORD = \"\"\n", hostname?hostname:"");
       if (((source_data) & (1<<(3)))>0)
         fprintf(file, "SOURCE_SSL = 1\n");
       else
@@ -640,13 +600,84 @@ void write_snapshot_info(MYSQL *conn, FILE *file) {
     g_message("Written master status");
   }
 
+  g_free(masterlog);
+  g_free(masterpos);
+  g_free(mastergtid);
+  fflush(file);
+}
+
+static
+void write_replica_info(MYSQL *conn, FILE *file) {
+  MYSQL_RES *slave = NULL;
+  MYSQL_FIELD *fields;
+  MYSQL_ROW row;
+
+  char *slavehost = NULL;
+  char *slavelog = NULL;
+  char *slavepos = NULL;
+  char *slavegtid = NULL;
+
+  char *channel_name = NULL;
+  const char *gtid_title = NULL;
+  guint i;
+  guint slave_count=0;
+  if (isms)
+    m_query_critical(conn, show_all_replicas_status, "Error executing %s", show_all_replicas_status);
+  else
+    m_query_critical(conn, show_replica_status, "Error executing %s", show_replica_status);
+
+  slave = mysql_store_result(conn);
+
+  GString *replication_section_str = g_string_sized_new(100);
+
+  while (slave && (row = mysql_fetch_row(slave))) {
+    g_string_set_size(replication_section_str,0);
+    fields = mysql_fetch_fields(slave);
+    slavepos=NULL;
+    slavelog=NULL;
+    slavehost=NULL;
+    slavegtid=NULL;
+    channel_name=NULL;
+    gtid_title=NULL;
+    for (i = 0; i < mysql_num_fields(slave); i++) {
+      if (!strcasecmp("exec_master_log_pos", fields[i].name)          || !strcasecmp("exec_source_log_pos", fields[i].name)) {
+        slavepos = row[i];
+      } else if (!strcasecmp("relay_master_log_file", fields[i].name) || !strcasecmp("relay_source_log_file", fields[i].name)) {
+        slavelog = row[i];
+      } else if (!strcasecmp("master_host", fields[i].name)           || !strcasecmp("source_host", fields[i].name)) {
+        slavehost = row[i];
+      } else if (!strcasecmp("Executed_Gtid_Set", fields[i].name)){
+        gtid_title="Executed_Gtid_Set";
+        slavegtid = remove_new_line(row[i]);
+      } else if (!strcasecmp("Gtid_Slave_Pos", fields[i].name)        || !strcasecmp("Gtid_source_Pos", fields[i].name)) {
+        gtid_title=g_strdup(fields[i].name);
+        slavegtid = remove_new_line(row[i]);
+      } else if ( ( !strcasecmp("connection_name", fields[i].name) || !strcasecmp("Channel_Name", fields[i].name) ) && strlen(row[i]) > 1) {
+        channel_name = row[i];
+      }
+      g_string_append_printf(replication_section_str,"# %s = ", fields[i].name);
+      (fields[i].type != MYSQL_TYPE_LONG && fields[i].type != MYSQL_TYPE_LONGLONG  && fields[i].type != MYSQL_TYPE_INT24  && fields[i].type != MYSQL_TYPE_SHORT )  ?
+      g_string_append_printf(replication_section_str,"'%s'\n", remove_new_line(row[i])):
+      g_string_append_printf(replication_section_str,"%s\n", remove_new_line(row[i]));
+    }
+    if (slavehost) {
+      slave_count++;
+      fprintf(file, "[replication%s%s]", channel_name!=NULL?".":"", channel_name!=NULL?channel_name:"");
+      fprintf(file, "\n# relay_master_log_file = \'%s\'\n# exec_master_log_pos = %s\n# %s = %s\n",
+              slavelog, slavepos, gtid_title, slavegtid);
+      fprintf(file,"%s",replication_section_str->str);
+      fprintf(file,"# myloader_exec_reset_slave = 0 # 1 means execute the command\n# myloader_exec_change_master = 0 # 1 means execute the command\n# myloader_exec_start_slave = 0 # 1 means execute the command\n");
+      g_message("Written slave status");
+    }
+  }
+  g_string_free(replication_section_str,TRUE);
+  if (slave_count > 1)
+    g_warning("Multisource replication found. Do not trust in the exec_master_log_pos as it might cause data inconsistencies. Search 'Replication and Transaction Inconsistencies' on MySQL Documentation");
+
   fflush(file);
 
-  if (master)
-    mysql_free_result(master);
-  if (mdb)
-    mysql_free_result(mdb);
-
+  if (slave)
+    mysql_free_result(slave);
 }
 
 static void thd_JOB_TABLE(struct thread_data *td, struct job *job);
@@ -666,6 +697,13 @@ gboolean process_job_builder_job(struct thread_data *td, struct job *job){
       break;
     case JOB_WRITE_MASTER_STATUS:
       write_snapshot_info(td->thrconn, job->job_data);
+      // Write replica information
+      if (get_product() != SERVER_TYPE_TIDB) {
+        if (source_data >=0 )
+          write_replica_info(td->thrconn, job->job_data);
+      }
+      g_async_queue_push(td->conf->binlog_ready,GINT_TO_POINTER(1));
+      g_async_queue_push(td->conf->binlog_ready,GINT_TO_POINTER(1));
       g_async_queue_push(td->conf->binlog_ready,GINT_TO_POINTER(1));
       g_free(job);
       break;
@@ -821,13 +859,9 @@ void *working_thread(struct thread_data *td) {
   execute_gstring(td->thrconn, set_session);
 
   // Initialize connection 
-  if (!skip_tz && mysql_query(td->thrconn, "/*!40103 SET TIME_ZONE='+00:00' */")) {
-    g_critical("Failed to set time zone: %s", mysql_error(td->thrconn));
-  }
-  if (use_savepoints && mysql_query(td->thrconn, "SET SQL_LOG_BIN = 0")) {
-    m_critical("Failed to disable binlog for the thread: %s",
-               mysql_error(td->thrconn));
-  }
+  if (!skip_tz)       m_query_critical(td->thrconn, "/*!40103 SET TIME_ZONE='+00:00' */", "Failed to set time zone", NULL);
+  if (use_savepoints) m_query_critical(td->thrconn, "SET SQL_LOG_BIN = 0", "Failed to disable binlog for the thread", NULL);
+
   initialize_consistent_snapshot(td);
   check_connection_status(td);
 
@@ -859,8 +893,9 @@ void *working_thread(struct thread_data *td) {
       g_async_queue_push(td->conf->unlock_tables, GINT_TO_POINTER(1));
     }else{
       // Sending LOCK TABLE over all non-transactional tables
-      if (td->conf->lock_tables_statement!=NULL && mysql_query(td->thrconn, td->conf->lock_tables_statement->str)) {
-        m_error("Error locking non-transactional tables %s", mysql_error(td->thrconn));
+      if (td->conf->lock_tables_statement!=NULL){
+        g_message("Thread %d: Locking non-transactional tables", td->thread_id);
+        m_query_critical(td->thrconn, td->conf->lock_tables_statement->str, "Error locking non-transactional tables", NULL);
       }
 
       // This push will unlock the FTWRL on the Main Connection
@@ -871,9 +906,7 @@ void *working_thread(struct thread_data *td) {
       process_queue(td->conf->non_transactional.defer, td, FALSE, NULL);
 
       // At this point, this thread is able to unlock the non-transactional tables
-      if (mysql_query(td->thrconn, UNLOCK_TABLES)) {
-        m_error("Error locking non-transactional tables %s", mysql_error(td->thrconn));
-      }
+      m_query_critical(td->thrconn, UNLOCK_TABLES, "Error locking non-transactional tables", NULL);
     }
 
     // Processing Transactional tables
@@ -884,10 +917,7 @@ void *working_thread(struct thread_data *td) {
     g_async_queue_push(td->conf->unlock_tables, GINT_TO_POINTER(1));
   }
 
-  if (use_savepoints && td->table_name != NULL &&
-      mysql_query(td->thrconn, "ROLLBACK TO SAVEPOINT mydumper")) {
-    g_critical("Rollback to savepoint failed: %s", mysql_error(td->thrconn));
-  }
+  if (use_savepoints && td->table_name != NULL) m_query_critical(td->thrconn, "ROLLBACK TO SAVEPOINT mydumper", "Rollback to savepoint failed", NULL);
 
   g_message("Thread %d: Processing remaining objects jobs", td->thread_id);
   process_queue(td->conf->post_data_queue, td, FALSE, NULL);
@@ -905,7 +935,6 @@ void *working_thread(struct thread_data *td) {
 
 static
 GString *get_selectable_fields(MYSQL *conn, char *database, char *table) {
-  MYSQL_RES *res = NULL;
   MYSQL_ROW row;
 
   GString *field_list = g_string_new("");
@@ -915,10 +944,9 @@ GString *get_selectable_fields(MYSQL *conn, char *database, char *table) {
                       "where TABLE_SCHEMA='%s' and TABLE_NAME='%s' and extra "
                       "not like '%%VIRTUAL GENERATED%%' and extra not like '%%STORED GENERATED%%' ORDER BY ORDINAL_POSITION ASC",
                       database, table);
-  mysql_query(conn, query);
+  MYSQL_RES *res=m_store_result_critical(conn, query, "Failed to get Selectable Fields", NULL);
   g_free(query);
 
-  res = mysql_store_result(conn);
   gboolean first = TRUE;
   while ((row = mysql_fetch_row(res))) {
     if (first) {
@@ -933,7 +961,7 @@ GString *get_selectable_fields(MYSQL *conn, char *database, char *table) {
     g_string_append(field_list, tb);
     g_free(tb);
   }
-//  g_string_append(field_list, ")");
+
   mysql_free_result(res);
 
   return field_list;
@@ -941,59 +969,37 @@ GString *get_selectable_fields(MYSQL *conn, char *database, char *table) {
 
 static
 gboolean has_json_fields(MYSQL *conn, char *database, char *table) {
-  MYSQL_RES *res = NULL;
-  MYSQL_ROW row;
-
   gchar *query =
       g_strdup_printf("select COLUMN_NAME from information_schema.COLUMNS "
                       "where TABLE_SCHEMA='%s' and TABLE_NAME='%s' and "
                       "COLUMN_TYPE ='json'",
                       database, table);
-  mysql_query(conn, query);
+  struct M_ROW *mr = m_store_result_row(conn, query, m_critical, m_warning, "Failed to get JSON fields on %s.%s: %s", database, table, query);
   g_free(query);
-
-  res = mysql_store_result(conn);
-  if (res == NULL){
-    return FALSE;
-  }
-  row = mysql_fetch_row(res);
-  if (row != NULL){
-    mysql_free_result(res);
+  if (mr->row){
+    m_store_result_row_free(mr);
     return TRUE;
   }
-//  g_string_append(field_list, ")");
-  mysql_free_result(res);
-
+  m_store_result_row_free(mr);
   return FALSE;
 }
 
 static
 gboolean detect_generated_fields(MYSQL *conn, gchar *database, gchar* table) {
-  MYSQL_RES *res = NULL;
-  MYSQL_ROW row;
-
   gboolean result = FALSE;
   if (ignore_generated_fields)
     return FALSE;
 
   gchar *query = g_strdup_printf(
-      "select COLUMN_NAME from information_schema.COLUMNS where "
-      "TABLE_SCHEMA='%s' and TABLE_NAME='%s' and extra like '%%GENERATED%%' and extra not like '%%DEFAULT_GENERATED%%'",
+      "SELECT COLUMN_NAME FROM information_schema.COLUMNS "
+      "WHERE TABLE_SCHEMA='%s' AND TABLE_NAME='%s' AND extra LIKE '%%GENERATED%%' AND extra NOT LIKE '%%DEFAULT_GENERATED%%' "
+      "LIMIT 1",
       database, table);
 
-  mysql_query(conn, query);
+  struct M_ROW *mr = m_store_result_row(conn, query, m_warning, m_message, "Failed to detect generated fields", NULL);
   g_free(query);
-
-  res = mysql_store_result(conn);
-  if (res == NULL){
-    return FALSE;
-  }
-
-  if ((row = mysql_fetch_row(res))) {
-    result = TRUE;
-  }
-  mysql_free_result(res);
-
+  result=mr->row!=NULL;
+  m_store_result_row_free(mr);
   return result;
 }
 
@@ -1002,19 +1008,15 @@ gchar *get_character_set_from_collation(MYSQL *conn, gchar *collation){
   g_mutex_lock(character_set_hash_mutex);
   gchar *character_set = g_hash_table_lookup(character_set_hash, collation);
   if (character_set == NULL){
-    MYSQL_RES *res = NULL;
-    MYSQL_ROW row;
     gchar *query =
       g_strdup_printf("SELECT CHARACTER_SET_NAME FROM INFORMATION_SCHEMA.COLLATIONS "
                       "WHERE collation_name='%s'",
                       collation);
-    mysql_query(conn, query);
+    struct M_ROW *mr = m_store_result_row(conn, query, m_critical, m_warning, "Failed to get CHARACTER_SET from collation %s", collation);
     g_free(query);
-    res = mysql_store_result(conn);
-    row = mysql_fetch_row(res);
-    if (row)
-      g_hash_table_insert(character_set_hash, g_strdup(collation), character_set=g_strdup(row[0]));
-    mysql_free_result(res);
+    if (mr->row)
+      g_hash_table_insert(character_set_hash, g_strdup(collation), character_set=g_strdup(mr->row[0]));
+    m_store_result_row_free(mr);
   }
   g_mutex_unlock(character_set_hash_mutex);
   return character_set;
@@ -1098,6 +1100,8 @@ gboolean new_db_table(struct db_table **d, MYSQL *conn, struct configuration *co
       g_warning("Setting min and start rows per file to 2 on %s", lkey);
     }
     dbt->is_fixed_length=dbt->min_chunk_step_size != 0 && dbt->min_chunk_step_size == dbt->starting_chunk_step_size && dbt->starting_chunk_step_size == dbt->max_chunk_step_size;
+    // We are disabling the file size limit when the user set a specific --rows size
+    dbt->chunk_filesize=dbt->is_fixed_length?0:chunk_filesize;
 
     if ( dbt->min_chunk_step_size==0)
       dbt->min_chunk_step_size=MIN_CHUNK_STEP_SIZE;
@@ -1113,15 +1117,11 @@ gboolean new_db_table(struct db_table **d, MYSQL *conn, struct configuration *co
     dbt->estimated_remaining_steps=1;
     dbt->min=NULL;
     dbt->max=NULL;
-//  dbt->chunk_type_item.chunk_type = UNDEFINED;
-//  dbt->chunk_type_item.chunk_step = NULL;
     dbt->chunks=NULL;
-//  dbt->initial_chunk_step=NULL;
     dbt->load_data_header=NULL;
     dbt->load_data_suffix=NULL;
     dbt->insert_statement=NULL;
     dbt->chunks_mutex=g_mutex_new();
-//  g_mutex_lock(dbt->chunks_mutex);
     dbt->chunks_queue=g_async_queue_new();
     dbt->chunks_completed=g_new(int,1);
     *(dbt->chunks_completed)=0;
@@ -1131,10 +1131,6 @@ gboolean new_db_table(struct db_table **d, MYSQL *conn, struct configuration *co
       get_primary_key_separated_by_comma(dbt);
     dbt->multicolumn = !use_single_column && g_list_length(dbt->primary_key) > 1;
 
-//  dbt->primary_key = get_primary_key_string(conn, dbt->database->name, dbt->table);
-    dbt->chunk_filesize=chunk_filesize;
-//  create_job_to_determine_chunk_type(dbt, g_async_queue_push, );
-
     gchar *columns_on_select=g_hash_table_lookup(conf_per_table.all_columns_on_select_per_table, lkey);
 
     dbt->columns_on_insert=g_hash_table_lookup(conf_per_table.all_columns_on_insert_per_table, lkey);
@@ -1268,15 +1264,10 @@ gboolean determine_if_schema_is_elected_to_dump_post(MYSQL *conn, struct databas
     g_assert(nroutines > 0);
     for (guint r= 0; r < nroutines; r++) {
       query= g_strdup_printf("SHOW %s STATUS WHERE CAST(Db AS BINARY) = '%s'", routine_type[r], database->escaped);
-      if (mysql_query(conn, (query))) {
-        g_critical("Error showing procedure on: %s - Could not execute query: %s", database->name,
-                  mysql_error(conn));
-        errors++;
-        g_free(query);
-        return FALSE;
-      }
+      result=m_store_result(conn, query, m_critical, "Error showing procedure on: %s - Could not execute query", database->name);
       g_free(query);
-      result= mysql_store_result(conn);
+      if (!result)
+        return FALSE;
       while ((row= mysql_fetch_row(result))) {
         /* Checks skip list on 'database.sp' string */
         if (tables_skiplist_file && check_skiplist(database->name, row[1]))
@@ -1296,15 +1287,10 @@ gboolean determine_if_schema_is_elected_to_dump_post(MYSQL *conn, struct databas
   if (dump_events) {
     // EVENTS
     query = g_strdup_printf("SHOW EVENTS FROM %s%s%s", identifier_quote_character_str, database->name, identifier_quote_character_str);
-    if (mysql_query(conn, (query))) {
-      g_critical("Error showing events on: %s - Could not execute query: %s", database->name,
-                 mysql_error(conn));
-      errors++;
-      g_free(query);
-      return FALSE;
-    }
+    result=m_store_result(conn, query, m_critical, "Error showing events on: %s - Could not execute query", database->name);
     g_free(query);
-    result = mysql_store_result(conn);
+    if (!result)
+      return FALSE;
     while ((row = mysql_fetch_row(result))) {
       /* Checks skip list on 'database.sp' string */
       if (tables_skiplist_file && check_skiplist(database->name, row[1]))
@@ -1324,29 +1310,20 @@ gboolean determine_if_schema_is_elected_to_dump_post(MYSQL *conn, struct databas
 static
 void dump_database_thread(MYSQL *conn, struct configuration *conf, struct database *database) {
 
-  const char *query= "SHOW TABLE STATUS";
-
   if (mysql_select_db(conn, database->name)) {
     g_critical("Could not select database: %s (%s)", database->name, mysql_error(conn));
     errors++;
     return;
   }
 
-  if (mysql_query(conn, (query))) {
-    g_critical("Error showing tables on: %s - Could not execute query: %s", database->name,
-               mysql_error(conn));
-    errors++;
+  const char *query= "SHOW TABLE STATUS";
+  MYSQL_RES *result = m_store_result(conn, query,m_critical, "Error showing tables on: %s - Could not execute query", database->name);
+  if (!result)
     return;
-  }
 
-  MYSQL_RES *result = mysql_store_result(conn);
   guint ecol= -1, ccol= -1, collcol= -1, rowscol= 0;
   determine_show_table_status_columns(result, &ecol, &ccol, &collcol, &rowscol);
-  if (!result) {
-    g_critical("Could not list tables for %s: %s", database->name, mysql_error(conn));
-    errors++;
-    return;
-  }
+
   guint i=0;
   MYSQL_ROW row;
   while ((row = mysql_fetch_row(result))) {
