@@ -72,21 +72,12 @@ gboolean hex_blob = FALSE;
 
 gboolean update_files_on_table_job(struct table_job *tj)
 {
-  struct chunk_step_item *csi= tj->chunk_step_item;
   if (tj->rows->file == 0){
-    if (csi->chunk_type == INTEGER) {
-      struct integer_step *s= &csi->chunk_step->integer_step;
-      if (s->is_step_fixed_length) {
-        tj->sub_part= (s->is_unsigned ? s->type.unsign.min : (guint64) s->type.sign.min) / s->step + 1;
-      }
-    }
-
-
-    tj->rows->filename = build_rows_filename(tj->dbt->database->filename, tj->dbt->table_filename, tj->nchunk, tj->sub_part);
+    tj->rows->filename = build_rows_filename(tj->dbt->database->filename, tj->dbt->table_filename, tj->part, tj->sub_part);
     tj->rows->file = m_open(&(tj->rows->filename),"w");
 
     if (tj->sql){
-      tj->sql->filename =build_sql_filename(tj->dbt->database->filename, tj->dbt->table_filename, tj->nchunk, tj->sub_part);
+      tj->sql->filename =build_sql_filename(tj->dbt->database->filename, tj->dbt->table_filename, tj->part, tj->sub_part);
       tj->sql->file = m_open(&(tj->sql->filename),"w");
       return TRUE;
     }
@@ -663,43 +654,47 @@ void update_dbt_rows(struct db_table * dbt, guint64 num_rows){
   g_mutex_unlock(dbt->rows_lock);
 }
 
-
-void initiliaze_load_data_files(struct table_job * tj, struct db_table * dbt){
-
-  m_close(tj->td->thread_id, tj->sql->file, g_strdup(tj->sql->filename), 1, dbt);
-  m_close(tj->td->thread_id, tj->rows->file, g_strdup(tj->rows->filename), 1, dbt);
-  tj->sql->file=0;
-  tj->rows->file=0;
-
-  g_free(tj->sql->filename);
-  g_free(tj->rows->filename);
-
-  tj->sql->filename=NULL;
-  tj->rows->filename=NULL;
-
-  if (update_files_on_table_job(tj)){
-    write_load_data_statement(tj);
-    write_header(tj);
+void close_file(struct table_job * tj, struct table_job_file *tjf){
+  if (tjf->file!=0){
+    m_close(tj->td->thread_id, tjf->file, tjf->filename, 1, tj->dbt);
+    tjf->file=0;
+    g_free(tjf->filename);
+    tjf->filename=NULL;
   }
 }
 
+void close_files(struct table_job * tj){
+  switch (output_format){
+    case LOAD_DATA:
+    case CSV:
+    case CLICKHOUSE:
+      close_file(tj, tj->sql);
+      break;
+    case SQL_INSERT:
+      break;
+  }
+  close_file(tj, tj->rows);
+}
 
-void initiliaze_clickhouse_files(struct table_job * tj, struct db_table * dbt){
-
-  m_close(tj->td->thread_id, tj->sql->file, g_strdup(tj->sql->filename), 1, dbt);
-  m_close(tj->td->thread_id, tj->rows->file, g_strdup(tj->rows->filename), 1, dbt);
-  tj->sql->file=0;
-  tj->rows->file=0;
-
-  g_free(tj->sql->filename);
-  g_free(tj->rows->filename);
-
-  tj->sql->filename=NULL;
-  tj->rows->filename=NULL;
-
-  if (update_files_on_table_job(tj)){
-    write_clickhouse_statement(tj);
-    write_header(tj);
+void reopen_files(struct table_job * tj){
+  close_files(tj);
+  switch (output_format){
+    case LOAD_DATA:
+    case CSV:
+      if (update_files_on_table_job(tj)){
+        write_load_data_statement(tj);
+        write_header(tj);
+      }
+      break;
+    case CLICKHOUSE:
+      if (update_files_on_table_job(tj)){
+        write_clickhouse_statement(tj);
+        write_header(tj);
+      }
+      break;
+    case SQL_INSERT:
+      update_files_on_table_job(tj);
+      break;
   }
 }
 
@@ -818,21 +813,10 @@ void write_result_into_file(MYSQL *conn, MYSQL_RES *result, struct table_job * t
 		if (dbt->chunk_filesize && (guint)ceil((float)tj->filesize / 1024 / 1024) >
               dbt->chunk_filesize){
 			tj->sub_part++;
-			switch (output_format){
-			  case LOAD_DATA:
-				case CSV:
-					initiliaze_load_data_files(tj, dbt);
-          break;
-				case CLICKHOUSE:
-  				initiliaze_clickhouse_files(tj, dbt);
-					break;
-				case SQL_INSERT:
-          m_close(tj->td->thread_id, tj->rows->file, tj->rows->filename, 1, dbt);
-          tj->rows->file=0;
-          update_files_on_table_job(tj);
-			  	initialize_sql_statement(tj->td->thread_data_buffers.statement);
-          g_string_append(tj->td->thread_data_buffers.statement, dbt->insert_statement->str);
-				  break;
+      reopen_files(tj);
+			if (output_format == SQL_INSERT){
+        initialize_sql_statement(tj->td->thread_data_buffers.statement);
+        g_string_append(tj->td->thread_data_buffers.statement, dbt->insert_statement->str);
       }
       tj->st_in_file = 0;
       tj->filesize = 0;			
@@ -855,6 +839,7 @@ void write_result_into_file(MYSQL *conn, MYSQL_RES *result, struct table_job * t
     }
 		tj->st_in_file++;
   }
+  tj->num_rows_of_last_run=num_rows;
 //  g_string_free(statement, TRUE);
 //  g_string_free(escaped, TRUE);
 //  g_string_free(statement_row, TRUE);	
@@ -864,13 +849,15 @@ void write_result_into_file(MYSQL *conn, MYSQL_RES *result, struct table_job * t
 /* Do actual data chunk reading/writing magic */
 void write_table_job_into_file(struct table_job * tj){
   MYSQL *conn = tj->td->thrconn;
-  MYSQL_RES *result = NULL;
   char *query = NULL;
 
+//  if (throttle_time)
+  g_usleep(throttle_time);
+
   /* Ghm, not sure if this should be statement_size - but default isn't too big
    * for now */
   /* Poor man's database code */
-  query = g_strdup_printf(
+  MYSQL_RES *result = m_use_result(conn, query = g_strdup_printf(
       "SELECT %s %s FROM %s%s%s.%s%s%s %s %s %s %s %s %s %s %s %s %s %s",
       is_mysql_like() ? "/*!40001 SQL_NO_CACHE */" : "",
       tj->dbt->select_fields?tj->dbt->select_fields->str:"*",
@@ -880,8 +867,9 @@ void write_table_job_into_file(struct table_job * tj){
       ((tj->where->len || where_option ) && tj->dbt->where) ? "AND"    : "" , tj->dbt->where ? tj->dbt->where : "",
       order_by_primary_key && tj->dbt->primary_key_separated_by_comma ? " ORDER BY " : "", order_by_primary_key && tj->dbt->primary_key_separated_by_comma ? tj->dbt->primary_key_separated_by_comma : "",
       tj->dbt->limit ?  "LIMIT" : "", tj->dbt->limit ? tj->dbt->limit : ""
-  );
-  if (mysql_query(conn, query) || !(result = mysql_use_result(conn))) {
+  ), m_warning, "Failed to execute query", NULL);
+
+  if (!result){
     if (!it_is_a_consistent_backup){
       g_warning("Thread %d: Error dumping table (%s.%s) data: %s\nQuery: %s", tj->td->thread_id, tj->dbt->database->name, tj->dbt->table,
                 mysql_error(conn), query);
@@ -892,7 +880,8 @@ void write_table_job_into_file(struct table_job * tj){
       }
       g_warning("Thread %d: Retrying last failed executed statement", tj->td->thread_id);
 
-      if (mysql_query(conn, query) || !(result = mysql_use_result(conn))) {
+      result = m_use_result(conn, query, NULL, "Failed to execute query on second try", NULL);
+      if (!result) {
        // ERROR 1146
         if (success_on_1146 && mysql_errno(conn) == 1146) {
           g_warning("Thread %d: Error dumping table (%s.%s) data: %s\nQuery: %s", tj->td->thread_id, tj->dbt->database->name, tj->dbt->table,
@@ -918,7 +907,7 @@ void write_table_job_into_file(struct table_job * tj){
   }
 
   /* Poor man's data dump code */
-    write_result_into_file(conn, result, tj);
+  write_result_into_file(conn, result, tj);
 
   if (mysql_errno(conn)) {
     g_critical("Thread %d: Could not read data from %s.%s to write on %s at byte %.0f: %s", tj->td->thread_id, tj->dbt->database->name, tj->dbt->table, tj->rows->filename, tj->filesize,
